#include "stdafx.h"
#include "classes.h"
#include "mapstuff.h"



void LoadSystemArea(FILE* rom, SystemArea *sys)
{
	for(int i = 0; i < 32; i++) {
		int address = GetPointer(rom, 0x2E9400+(i*0x10));

		sys->entries[i].index = i;

		if(address != 0) {
			sys->entries[i].is_valid = true;
			sys->entries[i].address = address;

			fseek(rom, 0x2E9400+(i*0x10)+4, SEEK_SET);

			sys->entries[i].size = fgetc(rom);
			sys->entries[i].size += fgetc(rom) << 8;

			sys->entries[i].data = new unsigned char[sys->entries[i].size];

			fseek(rom, sys->entries[i].address, SEEK_SET);

			for(int j = 0; j < sys->entries[i].size; j++) {
				sys->entries[i].data[j] = fgetc(rom);
			}
		}
		else
			sys->entries[i].is_valid = false;
	}
}

void WriteSystemArea(FILE* rom, SystemArea sys)
{
	int offset = 0;		// how much data we've already written

	for(int i = 0; i < 32; i++) {
		// if it's valid, we'll need to give it an address and write its data + pointer
		if(sys.entries[i].is_valid == true) {
			int address = 0x2E9600+offset;		// address to write data to
												// (0x2E9600 is the base for system data)

			// write pointer and size
			PutPointer(rom, 0x2E9400+i*0x10, address);

			fseek(rom, 0x2E9400+(i*0x10)+4, SEEK_SET);
			fputc(sys.entries[i].size & 255, rom);
			fputc((sys.entries[i].size >> 8) & 255, rom);

			// now write the data
			fseek(rom, address, SEEK_SET);
			for(int j = 0; j < sys.entries[i].size; j++) {
				fputc(sys.entries[i].data[j], rom);
			}
			offset+=sys.entries[i].size;
		}
	}
	fflush(rom);
}


void GetROMStuff(FILE* rom, int mt[320][256], int ls[320][256], int gs[80][32], int gp[80][32], int gm[80][32])
{
	int i, j;

	// map tiles
	for(i = 0; i < 320; i++) {
		fseek(rom, line_adr[i], SEEK_SET);
		for(j = 0; j < 256; j++) {
			mt[i][j] = fgetc(rom);
		}
	}

	// global tilesets
	for(i = 0; i < 80; i++) {
		fseek(rom, 0x17AA00+(i*32), SEEK_SET);
		for(j = 0; j < 32; j++) {
			int ch = fgetc(rom);
			gs[i][j] = (ch &248)>> 3;
			gp[i][j] = ch & 7;
		}
	}

	// music
	for(i = 0; i < 80; i++) {
		fseek(rom, 0x1CD837+(i*32), SEEK_SET);
		for(j = 0; j < 32; j++) {
			gm[i][j] = fgetc(rom);
		}
	}

	// local tilesets
	for(i = 0; i < 80; i++) {
		for(j = 0; j < 256; j++) {
			if(i%2==0)
				fseek(rom, 0x175200+(256*(i/2))+j, SEEK_SET);
			else
				fseek(rom, 0x178200+(256*(i/2))+j, SEEK_SET);
			char ch = fgetc(rom);

			int t1,t2,t3,t4;
			t4 = (ch >> 6) & 3;
			t3 = (ch >> 4) & 3;
			t2 = (ch >> 2) & 3;
			t1 = (ch /**/) & 3;

			int y = (i*4);
			int x = j;

			ls[y][x]   = t1%4;
			ls[y+1][x] = t2%4;
			ls[y+2][x] = t3%4;
			ls[y+3][x] = t4%4;
		}
	}
}



void SetROMStuff(FILE* rom, int mt[320][256], int ls[320][256], int gs[80][32], int gp[80][32], int gm[80][32])
{
	int i,j;

	// tile placement
	for(i = 0; i < 320; i++) {
		fseek(rom, line_adr[i], SEEK_SET);
		for(j = 0; j < 256; j++) {
			fputc(mt[i][j], rom);
		}
	}
	fflush(rom);

	// music
	for(i = 0; i < 80; i++) {
		fseek(rom, 0x1CD837+(i*32), SEEK_SET);
		for(j = 0; j < 32; j++) {
			fputc(gm[i][j],rom);
		}
	}
	fflush(rom);

	// global sets/palettes
	for(i = 0; i < 80; i++) {
		fseek(rom, 0x17AA00+(i*32), SEEK_SET);
		for(j = 0; j < 32; j++) {
			int ch = (gs[i][j]<<3) | gp[i][j];
			fputc(ch, rom);
		}
	}
	fflush(rom);

	// local tilesets
	for(i = 0; i < 80; i++) {
		for(j = 0; j < 256; j++) {
			if(i%2==0)
				fseek(rom, 0x175200+(256*(i/2))+j, SEEK_SET);
			else
				fseek(rom, 0x178200+(256*(i/2))+j, SEEK_SET);
			int ch = 0;

			int y = (i*4);
			int x = j;

			ch += ls[y][x];
			ch += ls[y+1][x] << 2;
			ch += ls[y+2][x] << 4;
			ch += ls[y+3][x] << 6;

			fputc(ch, rom);
		}
	}
	fflush(rom);
}




void AddDoor(ObjectArea *areas, int area, int x, int y, int flags, int dest, int destindex, int dir, int rope, int esc_end)
{
	DoorEntry *tempdoors = new DoorEntry[areas[area].num_doors];

	for(int i = 0; i < areas[area].num_doors; i++)
		tempdoors[i] = areas[area].doors[i];

	areas[area].doors = new DoorEntry[areas[area].num_doors+1];

	for(i = 0; i < areas[area].num_doors; i++)
		areas[area].doors[i] = tempdoors[i];

	int door = areas[area].num_doors;

	areas[area].doors[door].x = x;
	areas[area].doors[door].y = y;
	areas[area].doors[door].flags = flags;
	areas[area].doors[door].destination = dest;

	areas[area].doors[door].destindex = destindex;
	areas[area].doors[door].direction = dir;
	areas[area].doors[door].ropetype = rope;
	areas[area].doors[door].is_esc_end = esc_end;

	delete tempdoors;

	areas[area].num_doors++;
}

void RemDoor(ObjectArea *areas, int area, int door)
{
	DoorEntry *tempdoors = new DoorEntry[areas[area].num_doors];

	for(int i = 0; i < areas[area].num_doors; i++)
		tempdoors[i] = areas[area].doors[i];

	areas[area].num_doors--;

	for(i = door; i < areas[area].num_doors; i++)
		tempdoors[i] = tempdoors[i+1];

	areas[area].doors = new DoorEntry[areas[area].num_doors];

	for(i = 0; i < areas[area].num_doors; i++)
		areas[area].doors[i] = tempdoors[i];

	delete tempdoors;
}

void ChangeDoor(ObjectArea *areas, int area, int door, int x, int y, int flags, int dest)
{
	areas[area].doors[door].x = x;
	areas[area].doors[door].y = y;
	areas[area].doors[door].flags = flags;
	areas[area].doors[door].destination = dest;
}

void AddSprite(ObjectArea *areas, int area, int x, int y, int tptNum)
{
	SpriteEntry *tempsprites = new SpriteEntry[areas[area].num_sprites];

	for(int i = 0; i < areas[area].num_sprites; i++)
		tempsprites[i] = areas[area].sprites[i];

	areas[area].sprites = new SpriteEntry[areas[area].num_sprites+1];

	for(i = 0; i < areas[area].num_sprites; i++)
		areas[area].sprites[i] = tempsprites[i];

	int sprite = areas[area].num_sprites;

	areas[area].sprites[sprite].x = x;
	areas[area].sprites[sprite].y = y;
	areas[area].sprites[sprite].tptNum = tptNum;

	delete tempsprites;

	areas[area].num_sprites++;
}

void RemSprite(ObjectArea *areas, int area, int sprite)
{
	SpriteEntry *tempsprites = new SpriteEntry[areas[area].num_sprites];

	for(int i = 0; i < areas[area].num_sprites; i++)
		tempsprites[i] = areas[area].sprites[i];

	areas[area].num_sprites--;

	for(i = sprite; i < areas[area].num_sprites; i++)
		tempsprites[i] = tempsprites[i+1];

	areas[area].sprites = new SpriteEntry[areas[area].num_sprites];

	for(i = 0; i < areas[area].num_sprites; i++)
		areas[area].sprites[i] = tempsprites[i];

	delete tempsprites;
}

void ChangeSprite(ObjectArea *areas, int area, int sprite, int x, int y, int tptNum)
{
	areas[area].sprites[sprite].x = x;
	areas[area].sprites[sprite].y = y;
	areas[area].sprites[sprite].tptNum = tptNum;
}


void AddDestination(DestinationList *dest, int type, int pointer, int x, int y, int direction, int eventFlag, bool checkFlag, int style)
{
	// again, the entire array needs to be backed up during this operation
	DestinationEntry *temp = new DestinationEntry[dest->num_dest];

	for(int i = 0; i < dest->num_dest; i++)
		temp[i] = dest->entries[i];

	dest->entries = new DestinationEntry[dest->num_dest+1];

	for(i = 0; i < dest->num_dest; i++)
		dest->entries[i] = temp[i];

	dest->entries[dest->num_dest].type = type;
	dest->entries[dest->num_dest].pointer = pointer;
	dest->entries[dest->num_dest].x = x;
	dest->entries[dest->num_dest].y = y;
	dest->entries[dest->num_dest].direction = direction;
	dest->entries[dest->num_dest].eventFlag = eventFlag;
	dest->entries[dest->num_dest].checkFlag = checkFlag;
	dest->entries[dest->num_dest].style = style;

	dest->num_dest++;

	delete temp;

}

void RemoveDestination(DestinationList *dest, int destination, ObjectArea *areas)
{
	DestinationEntry *temp = new DestinationEntry[dest->num_dest];

	for(int i = 0; i < dest->num_dest; i++)
		temp[i] = dest->entries[i];

	dest->num_dest--;

	for(i = destination; i < dest->num_dest; i++)
		temp[i] = temp[i+1];

	dest->entries = new DestinationEntry[dest->num_dest];

	for(i = 0; i < dest->num_dest; i++)
		dest->entries[i] = temp[i];

	delete temp;

	// okay, we're not done yet.
	// here's the tricky part:
	// all the destination entries after the one we just deleted will get bumped back
	// one position. This also means that a very large number of door entries will
	// now have incorrect destination indexes attached to them.
	// basically, we have to find all the door entries that use those destinations above
	// the one we just deleted, and decrement their destination index values, so they'll
	// be in line with the new ones.
	// as for any doors that used the destination we just deleted?
	// heheh, well, they get set to zero.

	for(i = 0; i < 1280; i++) {
		for(int j = 0; j < areas[i].num_doors; j++) {
			if(areas[i].doors[j].destindex > destination)
				areas[i].doors[j].destindex--;

			if(areas[i].doors[j].destindex == destination) {
				areas[i].doors[j].destindex = 0;
				areas[i].doors[j].null_dest = true;
			}
		}
	}
}


void ChangeDestination(DestinationList *dest, int destination, int pointer, int x, int y, int eventFlag, bool checkFlag, int style)
{
	dest->entries[destination].pointer = pointer;
	dest->entries[destination].x = x;
	dest->entries[destination].y = y;
	dest->entries[destination].eventFlag = eventFlag;
	dest->entries[destination].checkFlag = checkFlag;
	dest->entries[destination].style = style;
}





int GetDETLength(ObjectArea *areas)
{
	int size = 0;
	for(int i = 0; i < 1280; i++) {
		size += 2;
		for(int j = 0; j < areas[i].num_doors; j++)
			size += 5;
	}

	return size;
}

int GetDestIndex(DestinationEntry *destinations, int num_dest, int destination)
{
	// retrieves the index of a given destination pointer.
	// destination data must be loaded prior to calling.
	// if a a destination matching the pointer is not found,
	// this returns -1, signifying that the object's destination
	// is indeterminate.
	int retVal = 0;

	for(int i = 0; i < num_dest; i++) {
		if(destinations[i].address == destination) {		// match found!
			retVal = i;
			break;
		}
	}

	return retVal;
}







bool CheckFile(CString szFile)
{
	// This function checks a file against various
	// criteria to make sure it is valid before loading.
	// There's the basic stuff, like making sure the file
	// can be opened and written to, or check to make sure
	// it's not a folder or some stupid thing like that.

	FILE* stream;
	char str[12];
	int idiot;

	stream = fopen(szFile, "rb+");

	if(stream) {
		fseek(stream, 0x0101C0, SEEK_SET);
		for(int i = 0; i < 11; i++)
			str[i] = fgetc(stream);
		str[11] = 0;
		if(strcmp(str, "EARTH BOUND"))
			idiot = AfxMessageBox("This doesn't look like an EarthBound ROM. Continuing may cause damage to your ROM. Do you want to continue anyway?", MB_YESNO | MB_ICONQUESTION);
		else
			return true;
		if(idiot == IDYES)
			return true;

		fclose(stream);
	}
	else {
		AfxMessageBox("An error occurred while attempting to open this file. Check to make sure the file is not read-only, and is not currently in use by another application.");
	}

	return false;
}


int CheckExpand(FILE* rom)
{
	int retVal = 0;
	int error;

	fseek(rom, 0, SEEK_END);
	long len = ftell(rom);

	if(len == 0x400200) {
		retVal = 1;
	}
	else if(len == 0x300200) {
		error = AfxMessageBox("WARNING\n\nThis ROM must be expanded before door data can be saved.\nDo you want to expand the ROM now?\n\n(Choosing \"No\" will skip saving door data)", MB_YESNOCANCEL);
		if(error == IDYES) {
			ExpandROM(rom);
			retVal = 1;
		}
		if(error == IDCANCEL) {
			retVal = -1;
		}
	}
	else {
		AfxMessageBox("ROM is unrecognized size.");
		retVal = -1;
	}

	return retVal;
}


int ExpandROM(FILE* rom)
{
	ZeroRange(rom, 0x300200, 0x400200, 1);
	return 1;
}



unsigned int HexToSNES(unsigned int address)
{
   // Converts good-ol regular hex addresses into the SNES-style addresses.
   // In this case, it's assumed the ROM is a hi-rom with a header.

   unsigned int retVal;

   retVal = (address + 0xc00000) - 0x200;

   return retVal;
}

unsigned int SNESToHex(unsigned int address)
{
   // Converts an SNES-style address (hi-rom, with header) to a nice
   // handy hex address.

   unsigned int retVal;

   retVal = (address - 0xc00000) + 0x200;

   return retVal;
}

unsigned int GetPointer(FILE* rom, unsigned int address)
{
	// Retrieves a 32-bit SNES pointer stored at address
	int curPos = ftell(rom);

	unsigned int retVal;

	fseek(rom, address, SEEK_SET);

	retVal =  fgetc(rom);
	retVal += fgetc(rom) << 8;
	retVal += fgetc(rom) << 16;

	fseek(rom, curPos, SEEK_SET);

	return retVal;
}

int PutPointer(FILE* rom, unsigned int address, unsigned int newPointer)
{
	// Write a 32-bit SNES pointer to address
	// NOTE: newPointer should be in SNES format, not standard hexadecimal
	// (Unless, of course, you _want_ to write a hex pointer into the ROM)
	// ...
	// (Moron.)
	int curPos = ftell(rom);

	fseek(rom, address, SEEK_SET);

	fputc(newPointer & 255, rom);
	fputc((newPointer >> 8) & 255, rom);
	fputc((newPointer >> 16) & 255, rom);

	fseek(rom, curPos, SEEK_SET);

	return 1;
}


CString FormatSNESAdr(int adr)
{
	// Formats a raw integer into a hex-format string with the
	// little '$' sign in front of it
	CString str = "                        ";
	char temp[24];

	itoa(adr, temp, 16);

	for(int i = 0; i < 24; i++)
		str.SetAt(i, temp[i]);

	str.Insert(0,'$');
	str.MakeUpper();

	return str;
}

int DeformatSNESAdr(CString adr)
{
	// The opposite of above; takes a formatted address and converts
	// it into the actual integer address
	int result;

	adr.Remove('$');

	result = strtoul((LPCSTR)adr, NULL, 16);

	return result;
}


unsigned long FindFreeRange(FILE* rom, unsigned long startAt, int length)
{
	// simple recursive function
	// finds the highest occurrence of a data-free block of size [length]
	// checks for normal expanded area data (i.e., 255 [00]s, one [02], and so on)
	// if it finds an interrupt in the pattern, it will skip that area and
	// search just beyond it, thus leaving any user-modified data intact (hopefully)

	int ch;

	for(unsigned long i = startAt; i > startAt-length; i--) {
		fseek(rom, i, SEEK_SET);

		ch = fgetc(rom);

		if(((i+1)%0x100) == 0) {
			if(ch != 2) {
				return FindFreeRange(rom, i-1, length);
			}
		}
		else {
			if(ch != 0) {
				return FindFreeRange(rom, i-1, length);
			}
		}
	}

	return i;
}

int ZeroRange(FILE* rom, unsigned int start, unsigned int end, int type)
{
	// Zeroes out a specified range with the specified range type
	// 0: Normal; all zeroes
	// 1: Text: 02's every 255 bytes

	int curPos = ftell(rom);

	fseek(rom, start, SEEK_SET);

	for(unsigned int i = start; i < end; i++) {
		if(type == 1 && ((i+1)%0x100) == 0)
			fputc(2, rom);
		else
			fputc(0, rom);
	}

	fseek(rom, curPos, SEEK_SET);

	return 1;
}





/*void LoadDoorData(FILE* rom)
{
	LoadDoorDestinations(rom, &dest);
	LoadDoorEntries(rom, object_areas, FIRSTLOAD);

	for(int i = 0; i < 1280; i++) {
		for(int j = 0; j < object_areas[i].num_doors; j++) {
			object_areas[i].doors[j].destindex = GetDestIndex(dest.entries, dest.num_dest, object_areas[i].doors[j].destination);
		}
	}
}*/




/*void SaveDoorData(FILE* rom, DestinationList dest)
{
	if(FIRSTLOAD) {
		// delete old stuff
		ZeroRange(rom, 0x0F0200, 0x0F284E, 0);	// destination table
	}
	ZeroRange(rom, det_first, det_last, (HexToSNES(det_first) >= 0xF00000));

	WriteDoorDestinations(rom, dest);

	// okay, the old door data's been wiped out, and we've got the door destinations done
	// time to set the addresses for the door destinations now
	for(int i = 0; i < 1280; i++) {
		for(int j = 0; j < object_areas[i].num_doors; j++) {
			object_areas[i].doors[j].destination = dest.entries[object_areas[i].doors[j].destindex].address;
		}
	}

	int det_length = GetDETLength(object_areas);
	int write_adr = FindFreeRange(rom, 0x400000, det_length);

	WriteEntryTable(rom, object_areas, write_adr);

	CreateAppData();
	WriteSystemArea(rom, sys);
}*/





// graphics functions

void DrawTile(FILE* rom, long address, int xx, int yy, int pal, unsigned char sprite[80][64])
{
   unsigned char ch;
   int           i;
   int           y;
   char          sprLine[8][8];

   fseek(rom, address, SEEK_SET);

   for (y = 0; y < 8; y++)
   {
      ch = fgetc(rom);

      for (i = 0; i < 8; i++)
         sprLine[y][7 - i] = (ch & (1 << i)) >> i;

      ch = fgetc(rom);

      for (i = 0; i < 8; i++)
         sprLine[y][7 - i] += ((ch & (1 << i)) >> i) << 1;
   }

   for (y = 0; y < 8; y++)
   {
      ch = fgetc(rom);

      for (i = 0; i < 8; i++)
         sprLine[y][7 - i] += ((ch & (1 << i)) >> i) << 2;

      ch = fgetc(rom);

      for (i = 0; i < 8; i++)
         sprLine[y][7 - i] += ((ch & (1 << i)) >> i) << 3;
   }

   for (y = 0; y < 8; y++)
   {
      for (i = 0; i < 8; i++)
      {
         if (sprLine[y][i] == 0)
            sprite[yy + y][xx + i] = 0;
         else
            sprite[yy + y][xx + i] = /*(16 * pal) +*/ sprLine[y][i];
      }
   }
}

void GetSprite(FILE* rom, Sprite *sprite)//int spriteNum, unsigned char spr[80][64], int* width, int* height)
{
   // width and height and palette will have to be predetermined and then
   // selected in this function.  Calling functions do not require this info.
   // Same goes for address of each sprite.  This function just takes a
   // sprite "number", which is determined by where it appears in the rom.

   unsigned int   address   = sprite->address;
   int            offset    = 0;
   int            pal       = sprite->palette;
   int			  width		= sprite->width;
   int			  height	= sprite->height;

   //address = sprite[spriteNum].address;
   
   //*width = sprite[spriteNum].width;
   //*height = sprite[spriteNum].height;
   //pal = sprite[spriteNum].palette;

   for (int y = 0; y < 80; y++)
   {
      for (int x = 0; x < 64; x++)
      {
         sprite->spr[y][x] = 0;
      }
   }

{
   for (int a = 0; a < height; a++)
   {
      for (int i = 0; i < width; i++)
      {
         DrawTile(rom, address + offset, i * 8, a * 8, pal, sprite->spr);
         offset += 32;
      }
   }
}
}


void ChangeTile(FILE* rom, unsigned int loc, unsigned char newTile[8][8])
{
   unsigned char num;
   unsigned char num2;
   unsigned char num3;
   unsigned char num4;
   int i,j;

   fseek(rom, loc, SEEK_SET);
   fflush(rom);

   for (j = 0; j < 8; j++)
   {
      num = 0;
      num2 = 0;
      num3 = 0;
      num4 = 0;

      for (i = 0; i < 8; i++)
      {
         num += ((newTile[j][i] & 0x01) << (7 - i));
         num2 += (((newTile[j][i] & 0x02) >> 1) << (7 - i));
         num3 += (((newTile[j][i] & 0x04) >> 2) << (7 - i));
         num4 += (((newTile[j][i] & 0x08) >> 3) << (7 - i));
      }

      fseek(rom, loc, SEEK_SET);
      fputc(num, rom);
      fputc(num2, rom);

      fseek(rom, loc + 0x10, SEEK_SET);
      fputc(num3, rom);
      fputc(num4, rom);

      loc += 2;
   }
}

void ChangeSprite(FILE* rom, Sprite sprite, unsigned char newTile[80][64])
{
   // Assumes rom is in write binary format.
   // spriteNum is the raw sprite number to be edited.  This is the #
   // associated with the original sprite viewer program, and goes from 0
   // to 1143.
   // newTile is the new sprite info to be stored over the sprite you
   // indicate.  Width and height are automatically trimmed to the correct
   // size.

   // NOTE: I gotta do something about that static array here, there's one
   // in the getsprite routine too, and that's A LOT of wasted memory,
   // plus creating static variables like this is bad design.  I'll figure
   // something out later.

   unsigned char temp[8][8];
   int           offset = 0;
   int           sx;
   int           sy;
   unsigned int  loc	= sprite.address;
   unsigned char width	= sprite.width;
   unsigned char height	= sprite.height;
   int y,x,iy,ix;

   for (y = 0; y < height; y++)
   {
      for (x = 0; x < width; x++)
      {
         sy = y * 8;
         sx = x * 8;
         
         for (iy = sy; iy < sy + 8; iy++)
         {
            for (ix = sx; ix < sx + 8; ix++)
            {
               temp[iy - sy][ix - sx] = newTile[iy][ix];
            }
         }

         ChangeTile(rom, loc + offset, temp);
         offset += 32;
      }
   }
}

void GetSpriteRGB(FILE* rom, int pal, int r[16], int g[16], int b[16])
{
   int           bgrBlock;
   unsigned char ch;
   unsigned char ch2;

   // the palette at 0x30200 is the first one, and is the one used
   // by human enemies to make their faces blue.
   // i think #8 could be the moonside neony palette

   fseek(rom, 0x30200 + pal * 32, SEEK_SET);

   ch = fgetc(rom);		// the first two bytes are insignificant, I guess
   ch = fgetc(rom);

   r[0] = 0;
   g[0] = 0;
   b[0] = 0;

   int i;
   for (i = 1; i < 16; i++)
   {
      ch = fgetc(rom);
      ch2 = fgetc(rom);

      bgrBlock = ch2 << 8;		// shift the second byte eight bits to the left
      bgrBlock += ch;			// add the first byte
      bgrBlock &= 0x7FFF;		// make sure the value does not exceed 7FFF?

      b[i] = bgrBlock >> 10;
      g[i] = (bgrBlock & 0x03e0) >> 5;
      r[i] = bgrBlock & 0x001f;

	  r[i]++;
	  g[i]++;
	  b[i]++;

      b[i] *= 8;				// multiply by eight to put the value into a 0-255 range
      g[i] *= 8;
      r[i] *= 8;

	  r[i]--;
	  g[i]--;
	  b[i]--;
   }
}

void SetSpriteRGB(FILE* rom, int pal, int r[16], int g[16], int b[16])
{
   int           bgrBlock;
   int cr,cg,cb;

   // the palette at 0x30200 is the first one, and is the one used
   // by human enemies to make their faces blue.
   // i think #8 could be the moonside neony palette

   fseek(rom, (0x30200 + pal * 32)+2, SEEK_SET);

   for (int i = 1; i < 16; i++)
   {
	  cr = r[i];
	  cg = g[i];
	  cb = b[i];

	  cb <<= 10;
	  cg <<= 5;

	  bgrBlock = cb;
	  bgrBlock += cg;
	  bgrBlock += cr;

	  fputc(bgrBlock & 255, rom);
	  fputc((bgrBlock >> 8) & 255,rom);
   }
   fflush(rom);
}




void InitOSpriteArray(Sprite sprites[1144])
{
	for(int i = 0; i < 1144; i++) {
		sprites[i].address = spr_address[i];
		sprites[i].width = spr_width[i];
		sprites[i].height = spr_height[i];
		sprites[i].palette = spr_palette[i];
	}
}


void LoadSprites(FILE* rom, Sprite *sprite, Palette *pal)
{
	InitOSpriteArray(sprite);

	for(int i = 0; i < 1144; i++) {
		GetSprite(rom, &sprite[i]);
	}
	for(i = 0; i < 8; i++) {
		GetSpriteRGB(rom, i, pal[i].r, pal[i].g, pal[i].b);
	}
}


void Sprite::Draw(CDC *pDC, int x, int y, Palette pal)
{
	for(int i = 0; i < height*8; i++)
	{
		for(int j = 0; j < width*8; j++)
		{
			int c = spr[i][j];

			if(c != 0) {
				pDC->SetPixel(x+j, y+i, RGB(pal.r[c], pal.g[c], pal.b[c]));
			}
		}
	}
}




void LoadTPT(FILE* rom, TPTEntry *tpt)
{
	int offset = 0;

	for(int i = 0; i < 1583; i++) {
		fseek(rom, 0xF8B96+offset, SEEK_SET);

		tpt[i].type = fgetc(rom);

		tpt[i].sprite = fgetc(rom);
		tpt[i].sprite+= fgetc(rom) << 8;

		tpt[i].dir = fgetc(rom) >> 1;

		tpt[i].movement1 = fgetc(rom);
		tpt[i].movement2 = fgetc(rom);

		tpt[i].flag = fgetc(rom);
		tpt[i].flag+= fgetc(rom) << 8;

		tpt[i].flaguse = fgetc(rom);

		tpt[i].pointer = fgetc(rom);
		tpt[i].pointer+= fgetc(rom) << 8;
		tpt[i].pointer+= fgetc(rom) << 16;

		fgetc(rom);		// null pointer byte

		if(tpt[i].type == 2) {	// item
			tpt[i].item = fgetc(rom);

			tpt[i].itempointer = 0;
		}
		else {
			tpt[i].item = 0;

			tpt[i].itempointer = fgetc(rom);
			tpt[i].itempointer+= fgetc(rom) << 8;
			tpt[i].itempointer+= fgetc(rom) << 16;
		}

		offset+= 0x11;
	}
}

void WriteTPT(FILE* rom, TPTEntry *tpt)
{
	int offset = 0;

	for(int i = 0; i < 1583; i++) {
		fseek(rom, 0xF8B96+offset, SEEK_SET);

		fputc(tpt[i].type, rom);

		fputc(tpt[i].sprite & 255, rom);
		fputc((tpt[i].sprite >> 8) & 255, rom);

		fputc(tpt[i].dir<<1, rom);

		fputc(tpt[i].movement1, rom);
		fputc(tpt[i].movement2, rom);

		fputc(tpt[i].flag & 255, rom);
		fputc((tpt[i].flag >> 8) & 255, rom);
		fputc(tpt[i].flaguse, rom);

		fputc(tpt[i].pointer & 255, rom);
		fputc((tpt[i].pointer >> 8) & 255, rom);
		fputc((tpt[i].pointer >> 16) & 255, rom);
		fputc(0, rom);

		if(tpt[i].type == 2) {
			fputc(tpt[i].item, rom);
		}
		else {
			fputc(tpt[i].itempointer & 255, rom);
			fputc((tpt[i].itempointer >> 8) & 255, rom);
			fputc((tpt[i].itempointer >> 16) & 255, rom);
			fputc(0, rom);
		}

		offset+= 0x11;
	}
}