/////////////////////////////////////////////////////////////////////////////////////////////////
// Classes.h: declarations for data structures

#if !defined(CLASSES_INCLUDED)
#define CLASSES_INCLUDED

#include "ospr.h"
#include "spriteconv.h"

#define DEST_SWITCH 0
#define DEST_ROPE 1
#define DEST_DOOR 2
#define DEST_ESCALATOR 3
#define DEST_STAIR 4
#define DEST_OBJECT 5
#define DEST_PERSON 6

struct SystemEntry {
	bool is_valid;			// Validity

	int index;		// Ordering

	long address;	// Location of data
	int size;		// The amount of space necessary for this entry (in bytes)

	unsigned char *data;		// Should be allocated with length size
};


struct SystemArea {
	SystemEntry entries[32];
};


struct DoorEntry {
	unsigned int address;

	int x;
	int y;

	int destination;
	int destindex;

	int direction;

	int ropetype;

	int is_esc_end;

	bool null_dest;

	int flags;
};

struct SpriteEntry {
	unsigned int address;

	int x;
	int y;

	int tptNum;
};

// this struct is used to track information about which sprites are currently on-screen.
// this is used so we can tell which sprite the user has clicked on, etc.
struct DISPSPRINFO {
	int x, y, w, h;
	int area, spr;
};

struct ObjectArea {
	unsigned int door_address;
	unsigned int spr_address;

	int num_doors;
	DoorEntry *doors;

	int num_sprites;
	SpriteEntry *sprites;
};

struct DestinationEntry {
	int address;	// used for index locating

	int type;

	int x;
	int y;

	int direction;

	int pointer;

	int eventFlag;
	bool checkFlag;		// last bit of event data

	int style;
};

struct DestinationList {
	DestinationEntry *entries;
	int num_dest;
};



struct TPTEntry {
	int type;

	int sprite;

	int dir;
	int movement1;
	int movement2;

	int flag;
	int flaguse;

	int pointer;
	int itempointer;

	int item;
};

struct Palette {
	int r[16];
	int g[16];
	int b[16];
};

struct Sprite {
	unsigned int address;
	int width;
	int height;
	int palette;
	
	unsigned char spr[80][64];

	void Draw(CDC *pDC, int x, int y, Palette pal);
};


// functions



// data serialization
void GetROMStuff(FILE* rom, int[320][256],int[320][256],int[80][32],int[80][32],int[80][32]);
void SetROMStuff(FILE* rom, int[320][256],int[320][256],int[80][32],int[80][32],int[80][32]);

void LoadDoorEntries(FILE* rom, ObjectArea areas[1280], bool firstload);
void LoadSpriteEntries(FILE* rom, ObjectArea areas[1280], bool firstload);
void LoadDoorDestinations(FILE* rom, DestinationList *dest);

void WriteEntryTable(FILE* rom, ObjectArea *areas, unsigned int address);
void WriteSpriteTable(FILE* rom, ObjectArea *areas);
void WriteDoorDestinations(FILE* rom, DestinationList dest);

void LoadSystemArea(FILE* rom, SystemArea* sys);
void WriteSystemArea(FILE* rom, SystemArea sys);


// load-only accessory data

void DrawTile(FILE* rom, long address, int xx, int yy, int pal, unsigned char sprite[80][64]);
void GetSprite(FILE* rom, int spriteNum, unsigned char sprite[80][64], int* width, int* height);

void InitOSpriteArray(Sprite *sprites);

void GetSpriteRGB(FILE*, int, int[], int[], int[],int[],int[],int[]);
void SetSpriteRGB(FILE*, int, int[], int[], int[]);

void ChangeTile(FILE*, unsigned int, unsigned char[8][8]);
void ChangeSprite(FILE*, int, unsigned char[80][64]);

void LoadSprites(FILE* rom, Sprite *sprites, Palette *pal);

void LoadTPT(FILE* rom, TPTEntry *tpt);
void WriteTPT(FILE* rom, TPTEntry *tpt);

// data modification
void AddDoor(ObjectArea *areas, int area, int x, int y, int flags, int dest, int destindex, int dir, int rope, int esc_end);
void RemDoor(ObjectArea *areas, int area, int door);
void ChangeDoor(ObjectArea *areas, int area, int door, int x, int y, int flags, int dest);

void AddSprite(ObjectArea *areas, int area, int x, int y, int tptNum);
void RemSprite(ObjectArea *areas, int area, int sprite);
void ChangeSprite(ObjectArea *areas, int area, int sprite, int x, int y, int tptNum);

void AddDestination(DestinationList *dest, int type, int pointer, int x, int y, int direction, int eventFlag, bool checkFlag, int style);
void RemoveDestination(DestinationList *dest, int destination, ObjectArea *areas);
void ChangeDestination(DestinationList *dest, int destination, int pointer, int x, int y, int eventFlag, bool checkFlag, int style);

int GetDETLength(ObjectArea *areas);

void SaveDoorData();


// utilities
bool CheckFile(CString szFile);
int CheckExpand(FILE* rom);
int ExpandROM(FILE* rom);

unsigned int HexToSNES(unsigned int);
unsigned int SNESToHex(unsigned int);

CString FormatSNESAdr(int adr);
int DeformatSNESAdr(CString adr);

unsigned int GetPointer(FILE* rom, unsigned int);
int PutPointer(FILE* rom, unsigned int, unsigned int);

unsigned long FindFreeRange(FILE* rom, unsigned long startAt, int length);
int ZeroRange(FILE* rom, unsigned int start, unsigned int end, int type);

int GetDestIndex(DestinationEntry *destinations, int num_dest, int destination);

#endif