// MapEdit.cpp : Defines the class behaviors for the application.
//
// NOTE:
// Most of the source for this program is right here in this file,
// including the classes for the app's core, frame, and view.
// This is primarily to save myself the hassle of converting
// old code to a new architecture.
// In other words, it's much more convenient for me to have each
// class in the same scope as a handy set of global attributes.
// In other words, I completely butchered the MFC document/view
// architecture so I wouldn't have to change my existing data code.
// In retrospect, that was a really sloppy approach; but it works,
// so feh.

#include "stdafx.h"
#include <float.h>
#include "MapEdit.h"
#include "mapstuff.h"
#include "ToolsDlg.h"
#include "InfoDlg.h"
#include "SaveProgDlg.h"

#include "MainFrm.h"
#include "MapEditDoc.h"
#include "MapEditView.h"

#include "classes.h"
#include "destinationtypes.h"

#include "DestinationDlg.h"
#include "DoorDlg.h"
#include "SpriteDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// standard parameter definitions
#define SYSTEM_ID 3

#define UNDO_COUNT 1000

#define UNDO_SECTOR 0
#define UNDO_VALUE 1

#define TILE_PLACE 0
#define TILE_PICK 1
#define TILE_INFO 2
#define TILE_SELECT 3

#define MODE_MAP 0
#define MODE_DOOR 1
#define MODE_SPRITE 2

#define DOOR_NORMAL 0
#define DOOR_DESTSEEK 1
#define DOOR_ADD 2

#define SPRITE_NORMAL 0
#define SPRITE_ADD 1

#define SEEK_DESTDLGONLY 0
#define SEEK_DOORDLGFIRST 1


// additional structures

// contains information about tile changes
struct UndoInfo {
	int type;
	int offX;
	int offY;
	int width;
	int height;
	unsigned char tiles[4][8];
	unsigned char lsets[4][8];
};

struct Clipboard {
	int isValid;
	int isUseable;
	unsigned char sector[4][8];
	unsigned char tileset[4][8];
};



// color animation data
COLORREF anim_clrs[16] = {
	RGB(0,		0,	255),
	RGB(64,		0,	255),
	RGB(128,	0,	255),
	RGB(192,	0,	255),

	RGB(255,	0,	255),

	RGB(255,	0,	192),
	RGB(255,	0,	128),
	RGB(255,	0,	64),

	RGB(255,	0,	128),
	RGB(255,	0,	192),
	RGB(255,	0,	255),

	RGB(192,	0,	255),
	RGB(128,	0,	255),
	RGB(64,		0,	255),
	RGB(0,		0,	255),
	RGB(0,		0,	255)
};

COLORREF boxcolor;

int drawcount = 0;
int cur_frame = 0;
int num_frames = 16;
int anim_speed = 150;
int timer_update = 0;




/***********************************\
| global attributes and behavior	|
\***********************************/


// function declarations

void LoadDoorData(FILE* rom);

void CreateAppData();
void LoadAppData(FILE* rom);

void AddUndoState(int type, int &val_old);
void DoUndo();
void Copy();
void Paste();

BOOL IsCloseable();

void Load();
void Save();
void UpdateProg(int pos = -1);


// mode selection
int edit_mode = MODE_MAP;			// current editing mode
int tilemode = TILE_PLACE;			// current tile placement mode
int door_mode = DOOR_NORMAL;		// current door mode
int sprite_mode = SPRITE_NORMAL;



// ROM data
CString FileName;					// filename currently being edited
FILE* RomFile;						// one and only file object
BOOL is_loaded = FALSE;				// TRUE if a file is currently loaded

int map_tiles[320][256];			// tilemap
int local_sets[320][256];			// tileset data
int global_sets[80][32];			// ''
int global_pal[80][32];				// palette data
int global_music[80][32];			// music data

ObjectArea object_areas[1280];		// 8x8 areas for object placement

DestinationList dest;				// list of door destinations

SystemArea sys;						// system area
SystemEntry *appdata;				// application-specific system data


int det_first;						// address of start of door entry table
int det_last;						// address of end of door entry table
int ddt_last;						// address of end of door destination table

bool FIRSTLOAD;						// TRUE if ROM has not been edited with this app before


// graphics

Sprite sprites[1144];
Palette pal[8];

// misc non-editable data
TPTEntry tpt[1583];


// GUI data

int xOffset = 0;					// Scroll offset from map origin
int yOffset = 0;					//

int sector_x = -10;					// Coordinates of selected map sector
int sector_y = -10;					//

int place_tile = 0;					// Current tile for placement
int place_set = 0;					// Current tileset to use for placement

int mouse_x = 0;					// Current mouse position
int mouse_y = 0;					//

BOOL gridlines = TRUE;				// If TRUE, gridlines are drawn on the map

BOOL spriteborder = TRUE;			// If TRUE, borders are drawn around sprites

BOOL sprite_outlines_only = FALSE;	// If TRUE, only sprite borders are shown, not actual sprite

BOOL drag_sprite_outline = FALSE;	// If TRUE, only sprite border is shown during dragging

BOOL color_anim = TRUE;				// If TRUE, object borders flash

BOOL auto_update = TRUE;			// Edit box handler control

BOOL infomode = FALSE;				// TRUE if tile info dialog is displayed

int infoX;							// Position of tile to display info for
int infoY;							//


DISPSPRINFO disp_spr[1024];			// array of sprites currently on-screen
int n_disp_spr = 0;



// drag 'n' drop data
int drag_area = -1;					// the area that contains the door being dragged
int drag_door = -1;					// the door being dragged
int drag_sprite = -1;
int spr_offx = 0;
int spr_offy = 0;
BOOL is_door_dragging = FALSE;		// TRUE if a door is being dragged
BOOL is_sprite_dragging = FALSE;


// door selection data
int sel_area = -1;					// area containing selected door
int sel_door = -1;					// door selected within area
int sel_sprite = -1;
BOOL is_door_selected = FALSE;		// TRUE if a door is selected
BOOL is_sprite_selected = FALSE;




// clipboard/state memory stuff

Clipboard clipboard;				// app clipboard

UndoInfo undostates[UNDO_COUNT];	// array of actions performed
int actioncount = 0;				// number of actions performed

BOOL undoable = FALSE;				// TRUE if undo is available



// windows
CDialogBar *dlgBar;					// pointer to app's dialog bar

CToolsDlg toolwin;					// tileset window
CDoorDlg doorwin;					// door entry editor window
CDestinationDlg destwin;			// door destination editor window
CSpriteDlg spritewin;				// sprite entry editor window
CSaveProgDlg saveprogwin;			// load/save progress window

// bitmaps
CDC buffer;							// drawing buffer

CDC tilesets[32];					// map tiles

CDC shadow;							// gray shadow for overlay
CDC darkshadow;						// dark gray shadow for overlay
CDC crosshair;						// crosshair image for destination seeking


// rubber-band states
BOOL reopen_doordlg = FALSE;		// If TRUE, door editor is displayed after destination
									// editor is closed

BOOL react_gridlines = FALSE;		// If TRUE, gridlines are turned back on after destination
									// seeking









void LoadAppData(FILE* rom)
{
	// load the PK Hack system area into memory
	LoadSystemArea(rom, &sys);

	appdata = &(sys.entries[SYSTEM_ID]);

	if(!appdata->is_valid)
		FIRSTLOAD = true;
	else
		FIRSTLOAD = false;
}


void CreateAppData()
{
	// this should be called prior to WriteSystemArea

	appdata->is_valid = true;

	// write number of doors into system entry
	appdata->data = new unsigned char[dest.num_dest+16];
	appdata->size = dest.num_dest+16;	// first 16 bytes reserved

	appdata->data[0] = dest.num_dest & 255;
	appdata->data[1] = (dest.num_dest >> 8) & 255;

	for(int i = 0; i < dest.num_dest; i++) {
		appdata->data[i+16] = dest.entries[i].type;
	}
}





void LoadDoorEntries(FILE* rom, ObjectArea *areas, bool firstload)
{
	for(int i = 0; i < 1280; i++) {
		int address = SNESToHex(GetPointer(rom, 0x100200+(i*4)));
		int num_doors;

		fseek(rom, address, SEEK_SET);

		areas[i].door_address = address;

		num_doors = fgetc(rom);
		fgetc(rom);

		if(firstload) {
			switch(i) {
			case 49:
			case 81:
			case 97:
			case 98:
			case 140:
			case 300:
			case 333:
			case 395:
			case 562:
			case 604:
			case 613:
			case 681:
			case 688:
			case 752:
			case 815:
			case 816:
			case 911:
			case 1136:
				num_doors--;
				break;
			case 1073:
				num_doors-=2;
			}
		}

		areas[i].num_doors = num_doors;
		if(num_doors < 256 && num_doors >= 0) areas[i].doors = new DoorEntry[num_doors];	// allocate space for our door data

		for(int j = 0; j < num_doors; j++) {
			areas[i].doors[j].address = address + 2 + (j*5);
			areas[i].doors[j].y = fgetc(rom);
			areas[i].doors[j].x = fgetc(rom);
			areas[i].doors[j].flags = fgetc(rom);

			int ch = fgetc(rom);
			ch += fgetc(rom) << 8;

			areas[i].doors[j].destination =  ch;

			areas[i].doors[j].direction = (ch & 0x300) >> 8;

			areas[i].doors[j].ropetype = ch;

			areas[i].doors[j].is_esc_end = (ch == 0x8000);

		}
		UpdateProg();
	}
	det_first = areas[0].doors[0].address - 2;
	det_last = det_first+GetDETLength(areas);
}

void LoadSpriteEntries(FILE* rom, ObjectArea *areas, bool firstload)
{
	int address;
	int num_sprites;

	for(int i = 0; i < 1280; i++) {
		fseek(rom, 0xF63E7+(i*2), SEEK_SET);

		address = fgetc(rom);
		address += fgetc(rom) << 8;

		if(address == 0) continue;

		address += 0xF0200;

		fseek(rom, address, SEEK_SET);

		areas[i].spr_address = address;

		num_sprites = fgetc(rom);
		fgetc(rom);

		areas[i].num_sprites = num_sprites;
		if(num_sprites < 256 && num_sprites >= 0) areas[i].sprites = new SpriteEntry[num_sprites];

		for(int j = 0; j < num_sprites; j++) {
			areas[i].sprites[j].address = address + 2 + (j*5);

			areas[i].sprites[j].tptNum = fgetc(rom);
			areas[i].sprites[j].tptNum+= fgetc(rom) << 8;

			areas[i].sprites[j].tptNum--;

			areas[i].sprites[j].y = fgetc(rom);
			areas[i].sprites[j].x = fgetc(rom);
		}
	}
}


void WriteEntryTable(FILE* rom, ObjectArea *areas, unsigned int address)
{
	// writes the entry table to the specified address.
	// use FindFreeRange to find the starting address for this data

	// this function also modifies the pointer table as it writes
	// the door entry data.

	int offset = 0;

	// start with the standard loop
	for(int i = 0; i < 1280; i++) {
		PutPointer(rom, 0x100200+(i*4), HexToSNES(address+offset));

		fseek(rom, address+offset, SEEK_SET);

		fputc(areas[i].num_doors, rom);
		fputc(0, rom);

		for(int j = 0; j < areas[i].num_doors; j++) {
			areas[i].doors[j].address = address+offset+2;
			fputc(areas[i].doors[j].y, rom);
			fputc(areas[i].doors[j].x, rom);
			fputc(areas[i].doors[j].flags, rom);
			switch(areas[i].doors[j].flags) {
			case DEST_SWITCH:
			case DEST_DOOR:
			case DEST_OBJECT:
			case DEST_PERSON:
				fputc(areas[i].doors[j].destination & 255, rom);
				fputc((areas[i].doors[j].destination >> 8) & 255, rom);
				break;
			case DEST_ROPE:
				fputc(areas[i].doors[j].ropetype & 255, rom);
				fputc((areas[i].doors[j].ropetype >> 8) & 255, rom);
				break;
			case DEST_STAIR:
				fputc(0, rom);
				fputc((areas[i].doors[j].direction) & 255, rom);
				break;
			case DEST_ESCALATOR:
				fputc(0, rom);
				if(areas[i].doors[j].is_esc_end)
					fputc(0x80, rom);
				else
					fputc((areas[i].doors[j].direction) & 255, rom);
				break;
			}
			offset+=5;
		}
		offset+=2;

		UpdateProg();
	}
	det_first = areas[0].doors[0].address - 2;
	det_last = det_first+GetDETLength(areas);
}


void WriteSpriteTable(FILE* rom, ObjectArea *areas)
{
	// start writing sprites at 0x0F6DE7, and when we run out of room,
	// begin putting them at the end of the door entries
	int next_adr = ddt_last;

	int address = 0x0F6DE7;
	int offset = 0;

	for(int i = 0; i < 1280; i++)
	{
		// location that current sprite block will extend to
		int block_end = address+offset+2+(areas[i].num_sprites*4);

		if(block_end > 0x0F8B91) {
			address = next_adr;
			offset = 0;
		}

		// write pointer
		fseek(rom, 0x0F63E7+i*2, SEEK_SET);
		if(areas[i].num_sprites == 0) {
			fputc(0, rom);
			fputc(0, rom);
		}
		else {
			fputc((address+offset-0xF0200) & 255, rom);
			fputc(((address+offset-0xF0200) >> 8) & 255, rom);

			fseek(rom, address+offset, SEEK_SET);

			fputc(areas[i].num_sprites, rom);
			fputc(0, rom);
			offset+=2;

			for(int j = 0; j < areas[i].num_sprites; j++)
			{
				areas[i].sprites[j].address = address+offset+j*4;

				fputc((areas[i].sprites[j].tptNum+1) & 255, rom);
				fputc(((areas[i].sprites[j].tptNum+1) >> 8) & 255, rom);

				fputc(areas[i].sprites[j].y, rom);
				fputc(areas[i].sprites[j].x, rom);

				offset+=4;
			}
		}
	}
}


void LoadDoorDestinations(FILE* rom, DestinationList *dest)
{
	// door editor system area data must be loaded before this gets called.


	if(FIRSTLOAD) {
		dest->num_dest = 977;
	}
	else {
		dest->num_dest =  appdata->data[0];
		dest->num_dest += appdata->data[1] << 8;
	}

	// allocate space for destination table
	dest->entries = new DestinationEntry[dest->num_dest];

	int offset = 0;	//start at 0xF0200 + 0

	for(int i = 0; i < dest->num_dest; i++) {
		if(FIRSTLOAD) {
			// load current door type from a static array
			dest->entries[i].type = default_dest_types[i];
		}
		else {
			dest->entries[i].type = appdata->data[i+16];
		}

		fseek(rom, 0xF0200+offset, SEEK_SET);
		dest->entries[i].address = offset;
		
		// initialize values
		dest->entries[i].eventFlag = 0;
		dest->entries[i].checkFlag = 0;
		dest->entries[i].direction = 0;
		dest->entries[i].style = 0;
		dest->entries[i].x = 0;
		dest->entries[i].y = 0;
		dest->entries[i].pointer = 0;

		// now check stuff, do different things based on door's type.
		int ch;

		switch(dest->entries[i].type) {
		case DEST_DOOR:		// standard door. easy.
			dest->entries[i].pointer = fgetc(rom);
			dest->entries[i].pointer+= fgetc(rom) << 8;
			dest->entries[i].pointer+= fgetc(rom) << 16;
			fgetc(rom);

			dest->entries[i].eventFlag = fgetc(rom);

			ch = fgetc(rom);

			dest->entries[i].eventFlag+= (ch & 0x7F) << 8;
			dest->entries[i].checkFlag = (ch & 0x80) >> 7 != 0;


			dest->entries[i].y = fgetc(rom);

			ch = fgetc(rom);	// temp

			dest->entries[i].y+= (ch & 0x3F) << 8;
			dest->entries[i].direction = (ch & 0xC0) >> 6;


			dest->entries[i].x = fgetc(rom);
			dest->entries[i].x+= fgetc(rom) << 8;

			dest->entries[i].style = fgetc(rom);

			offset+= 11;
			break;

		case DEST_OBJECT: case DEST_PERSON:		// just text pointer
			dest->entries[i].pointer = fgetc(rom);
			dest->entries[i].pointer+= fgetc(rom) << 8;
			dest->entries[i].pointer+= fgetc(rom) << 16;

			offset+= 4;
			break;

		case DEST_SWITCH:	// event flag, pointer
			dest->entries[i].eventFlag = fgetc(rom);

			ch = fgetc(rom);

			dest->entries[i].eventFlag+= (ch & 0x7F) << 8;
			dest->entries[i].checkFlag = (ch & 0x80) >> 7 != 0;

			dest->entries[i].pointer = fgetc(rom);
			dest->entries[i].pointer+= fgetc(rom) << 8;
			dest->entries[i].pointer+= fgetc(rom) << 16;

			offset+= 6;
			break;
		}

		// update progess bar in loader window
		int pos = (1000/dest->num_dest)*i;
		UpdateProg(pos);
	}
}

void WriteDoorDestinations(FILE* rom, DestinationList dest)
{
	int ch;
	int offset = 0;

	for(int i = 0; i < dest.num_dest; i++) {
		fseek(rom, 0xF0200+offset, SEEK_SET);

		dest.entries[i].address = offset;

		switch(dest.entries[i].type) {
		case DEST_DOOR:
			fputc(dest.entries[i].pointer&255, rom);
			fputc((dest.entries[i].pointer >> 8) & 255, rom);
			fputc((dest.entries[i].pointer >> 16) & 255, rom);
			fputc(0, rom);

			ch = dest.entries[i].eventFlag + (dest.entries[i].checkFlag*0x8000);
			fputc(ch & 255, rom);
			fputc((ch >> 8) & 255, rom);

			ch = dest.entries[i].y;

			fputc(dest.entries[i].y & 255, rom);

			fputc(((dest.entries[i].y >> 8) | (dest.entries[i].direction) << 6) & 255, rom);

			fputc(dest.entries[i].x & 255, rom);
			fputc((dest.entries[i].x >> 8) & 255, rom);

			fputc(dest.entries[i].style, rom);

			offset+= 11;
			break;

		case DEST_OBJECT:
		case DEST_PERSON:
			fputc(dest.entries[i].pointer & 255, rom);
			fputc((dest.entries[i].pointer >> 8) & 255, rom);
			fputc((dest.entries[i].pointer >> 16) & 255, rom);
			fputc(0, rom);

			offset+= 4;
			break;

		case DEST_SWITCH:
			ch = dest.entries[i].eventFlag + (dest.entries[i].checkFlag*0x8000);
			fputc(ch & 255, rom);
			fputc((ch >> 8) & 255, rom);

			fputc(dest.entries[i].pointer & 255, rom);
			fputc((dest.entries[i].pointer >> 8) & 255, rom);
			fputc((dest.entries[i].pointer >> 16) & 255, rom);
			fputc(0, rom);

			offset+= 6;
			break;
		}
		UpdateProg();
	}

	ddt_last = 0xF0200+offset;	// end of destination table

	fflush(rom);
}



void UpdateProg(int pos)
{
	if(pos == -1)
		saveprogwin.m_Prog.StepIt();
	else
		saveprogwin.m_Prog.SetPos(pos);
}
void Load()
{
	// Load editable data
	saveprogwin.SetWindowText("Loading");

	saveprogwin.ShowWindow(SW_SHOWNORMAL);
	saveprogwin.m_Prog.SetRange(0, 2280);

	saveprogwin.SetDlgItemText(IDC_TEXT, "Loading...");

	GetROMStuff(RomFile, map_tiles, local_sets, global_sets, global_pal, global_music);

	LoadAppData(RomFile);

	LoadDoorDestinations(RomFile, &dest);
	LoadDoorEntries(RomFile, object_areas, FIRSTLOAD);

	for(int i = 0; i < 1280; i++) {
		for(int j = 0; j < object_areas[i].num_doors; j++) {
			object_areas[i].doors[j].destindex = GetDestIndex(dest.entries, dest.num_dest, object_areas[i].doors[j].destination);
		}
	}

	LoadSpriteEntries(RomFile, object_areas, FIRSTLOAD);


	// Load Graphics
	LoadSprites(RomFile, sprites, pal);

	for(i = 0; i < 1144; i++)
		spritewin.sprite_gfx[i] = sprites[i];

	for(i = 0; i < 8; i++)
		spritewin.pal[i] = pal[i];

	LoadTPT(RomFile, tpt);

	saveprogwin.ShowWindow(SW_HIDE);

}

void Save()
{
	saveprogwin.SetWindowText("Saving");
	saveprogwin.ShowWindow(SW_SHOWNORMAL);

	int total = 1280 + dest.num_dest;

	saveprogwin.m_Prog.SetRange(0, total);

	int save_flag = CheckExpand(RomFile);

	if(save_flag == 0 || save_flag == 1) {
		saveprogwin.SetDlgItemText(IDC_TEXT, "Saving map data...");
		SetROMStuff(RomFile, map_tiles, local_sets, global_sets, global_pal, global_music);
	}
	if(save_flag == 1) {
		if(FIRSTLOAD) {
			// delete old stuff
			ZeroRange(RomFile, 0x0F0200, 0x0F284E, 0);	// destination table
		}
		ZeroRange(RomFile, det_first, det_last, (HexToSNES(det_first) >= 0xF00000));


		saveprogwin.SetDlgItemText(IDC_TEXT, "Writing door destination table...");
		WriteDoorDestinations(RomFile, dest);


		// okay, the old door data's been wiped out, and we've got the door destinations done
		// time to set the addresses for the door destinations now
		for(int i = 0; i < 1280; i++) {
			for(int j = 0; j < object_areas[i].num_doors; j++) {
				object_areas[i].doors[j].destination = dest.entries[object_areas[i].doors[j].destindex].address;
			}
			//UpdateProg();
		}

		saveprogwin.SetDlgItemText(IDC_TEXT, "Allocating space...");
		int det_length = GetDETLength(object_areas);
		int write_adr = FindFreeRange(RomFile, 0x400000, det_length);

		saveprogwin.SetDlgItemText(IDC_TEXT, "Writing door entry table...");
		WriteEntryTable(RomFile, object_areas, write_adr);

		WriteSpriteTable(RomFile, object_areas);

		WriteTPT(RomFile, tpt);

		saveprogwin.SetDlgItemText(IDC_TEXT, "Writing system area...");
		CreateAppData();
		WriteSystemArea(RomFile, sys);
	}
	saveprogwin.ShowWindow(SW_HIDE);
}



void AddUndoState(int x, int y, int width, int height)
{
	for(int i = UNDO_COUNT-1; i > 0; i--)
		undostates[i] = undostates[i-1];

	undostates[0].offX = x;
	undostates[0].offY = y;
	undostates[0].width = width;
	undostates[0].height = height;

	for(i = 0; i < height; i++) {
		for(int j = 0; j < width; j++) {
			undostates[0].tiles[i][j] = map_tiles[y+i][x+j];
			undostates[0].lsets[i][j] = local_sets[y+i][x+j];
		}
	}

	actioncount++;
	undoable = TRUE;
}

void DoUndo()
{
	int x = undostates[0].offX;
	int y = undostates[0].offY;
	for(int i = 0; i < undostates[0].height; i++) {
		for(int j = 0; j < undostates[0].width; j++) {
			map_tiles[y+i][x+j] = undostates[0].tiles[i][j];
			local_sets[y+i][x+j] = undostates[0].lsets[i][j];
		}
	}
	actioncount--;

	for(i = 0; i < UNDO_COUNT-1; i++)
		undostates[i] = undostates[i+1];

	if(actioncount == 0)
		undoable = FALSE;
}

void Copy()
{
	clipboard.isValid = TRUE;
	for(int i = 0; i < 4; i++) {
		for(int j = 0; j < 8; j++) {
			clipboard.sector[i][j] = map_tiles[i+sector_y*4][j+sector_x*8];
			clipboard.tileset[i][j] = local_sets[i+sector_y*4][j+sector_x*8];
		}
	}
}

void Paste()
{
	AddUndoState(sector_x*8,sector_y*4,8,4);
	for(int i = 0; i < 4; i++) {
		for(int j = 0; j < 8; j++) {
			map_tiles[i+sector_y*4][j+sector_x*8] = clipboard.sector[i][j];
			local_sets[i+sector_y*4][j+sector_x*8] = clipboard.tileset[i][j]; 
		}
	}
}





/////////////////////////////////////////////////////////////////////////////
// CMapEditApp

BEGIN_MESSAGE_MAP(CMapEditApp, CWinApp)
	//{{AFX_MSG_MAP(CMapEditApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMapEditApp construction

CMapEditApp::CMapEditApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CMapEditApp object

CMapEditApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CMapEditApp initialization

BOOL CMapEditApp::InitInstance()
{
	AfxEnableControlContainer();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	// Change the registry key under which our settings are stored.
	// TODO: You should modify this string to be something appropriate
	// such as the name of your company or organization.
	SetRegistryKey(_T("EarthBound Map Editor"));

	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CSingleDocTemplate* pDocTemplate;
	pDocTemplate = new CSingleDocTemplate(
		IDR_MAINFRAME,
		RUNTIME_CLASS(CMapEditDoc),
		RUNTIME_CLASS(CMainFrame),       // main SDI frame window
		RUNTIME_CLASS(CMapEditView));
	AddDocTemplate(pDocTemplate);

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

	// The one and only window has been initialized, so show and update it.
	m_pMainWnd->ShowWindow(SW_SHOW);
	m_pMainWnd->UpdateWindow();

	return TRUE;
}





/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CMapEditApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CMapEditApp message handlers

CDocument* CMapEditApp::OpenDocumentFile(LPCTSTR lpszFileName) 
{
	((CMainFrame*)m_pMainWnd)->CloseFile();

	CDocument* temp = CWinApp::OpenDocumentFile(lpszFileName);

	if(CheckFile(lpszFileName)) {
		CMainFrame* mainfrm = (CMainFrame*)m_pMainWnd;
		mainfrm->LoadFile(lpszFileName);
	}
	
	return temp;

	//return CWinApp::OpenDocumentFile(lpszFileName);
}


void CMapEditApp::AddToRecentFileList(LPCTSTR lpszPathName) 
{
	//for(int i = 3; i > 1; i--) {
	//	MRU[i] = MRU[i-1];
	//}
	//MRU[0] = lpszPathName;

	CWinApp::AddToRecentFileList(lpszPathName);
}
















/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_CBN_SELCHANGE(IDC_COMBO_TILESET, OnSelectTileset)
	ON_EN_CHANGE(IDC_EDIT_X, OnEditX)
	ON_EN_CHANGE(IDC_EDIT_Y, OnEditY)
	ON_EN_CHANGE(IDC_EDIT_PALETTE, OnEditPalette)
	ON_EN_CHANGE(IDC_EDIT_MUSIC, OnEditMusic)
	ON_COMMAND(ID_VIEW_GRIDLINES, OnViewGridlines)
	ON_UPDATE_COMMAND_UI(ID_VIEW_GRIDLINES, OnUpdateViewGridlines)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_COMMAND(ID_FILE_CLOSE, OnFileClose)
	ON_UPDATE_COMMAND_UI(ID_FILE_CLOSE, OnUpdateFileClose)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	ON_COMMAND(ID_TOOLS_DESTINATIONEDIT, OnToolsDestination)
	ON_WM_TIMER()
	ON_COMMAND(ID_MODE_DOOR, OnModeDoor)
	ON_UPDATE_COMMAND_UI(ID_MODE_DOOR, OnUpdateModeDoor)
	ON_COMMAND(ID_MODE_MAP, OnModeMap)
	ON_UPDATE_COMMAND_UI(ID_MODE_MAP, OnUpdateModeMap)
	ON_COMMAND(ID_MODE_SPRITE, OnModeSprite)
	ON_UPDATE_COMMAND_UI(ID_MODE_SPRITE, OnUpdateModeSprite)
	ON_COMMAND(ID_TOOLS_ADDDOOR, OnToolsAddDoor)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_ADDDOOR, OnUpdateToolsAddDoor)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_DESTINATIONEDIT, OnUpdateToolsDestinationEdit)
	ON_COMMAND(ID_TOOLS_ADDSPRITE, OnToolsAddSprite)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_ADDSPRITE, OnUpdateToolsAddSprite)
	ON_COMMAND(ID_VIEW_SPRITEBORDERS, OnViewSpriteborders)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SPRITEBORDERS, OnUpdateViewSpriteborders)
	ON_COMMAND(ID_VIEW_SPRITEOUTLINES, OnViewSpriteOutlines)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SPRITEOUTLINES, OnUpdateViewSpriteOutlines)
	ON_COMMAND(ID_VIEW_DRAGSPRITEOUTLINE, OnViewDragSpriteOutline)
	ON_UPDATE_COMMAND_UI(ID_VIEW_DRAGSPRITEOUTLINE, OnUpdateViewDragSpriteOutline)
	ON_COMMAND(ID_VIEW_BORDERFLASH, OnViewBorderFlash)
	ON_UPDATE_COMMAND_UI(ID_VIEW_BORDERFLASH, OnUpdateViewBorderFlash)
	ON_COMMAND(ID_TOOLS_REMOVEOBJECT, OnToolsRemoveObject)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_REMOVEOBJECT, OnUpdateToolsRemoveObject)
	ON_COMMAND(ID_TOOLS_TPTEDITOR, OnToolsTPTEditor)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_TPTEDITOR, OnUpdateToolsTPTEditor)
	ON_COMMAND(ID_FILE_CLOSEROM, OnFileClose)
	ON_UPDATE_COMMAND_UI(ID_FILE_CLOSEROM, OnUpdateFileClose)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	// TODO: add member initialization code here
	
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}
	if (!m_wndDlgBar.Create(this, IDR_MAINFRAME, 
		CBRS_ALIGN_TOP, AFX_IDW_DIALOGBAR))
	{
		TRACE0("Failed to create dialogbar\n");
		return -1;		// fail to create
	}

	if (!m_wndReBar.Create(this) ||
		!m_wndReBar.AddBar(&m_wndToolBar) ||
		!m_wndReBar.AddBar(&m_wndDlgBar))
	{
		TRACE0("Failed to create rebar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// TODO: Remove this if you don't want tool tips
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY);

	// Load a bunch of fun stuff into our DCs
	buffer.CreateCompatibleDC(GetDC());
	SelectObject((HDC)buffer,CreateCompatibleBitmap((HDC)*GetDC(),1280,1024));

	for(int i = 0; i < 32; i++) {
		tilesets[i].CreateCompatibleDC(GetDC());
		HBITMAP bmp = (HBITMAP)LoadImage(NULL, filename[i], IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_CREATEDIBSECTION);
		if(bmp != NULL)
			SelectObject((HDC)tilesets[i], bmp);
		else {
			SelectObject((HDC)tilesets[i], CreateCompatibleBitmap((HDC)*GetDC(),8192,128));
			tilesets[i].FillSolidRect(0,0,8192,128,RGB(0,0,0));
		}
		DeleteObject(bmp);
	}
	shadow.CreateCompatibleDC(GetDC());
	darkshadow.CreateCompatibleDC(GetDC());
	SelectObject((HDC)shadow, CreateCompatibleBitmap((HDC)*GetDC(),1024,1024));
	SelectObject((HDC)darkshadow, CreateCompatibleBitmap((HDC)*GetDC(),1024,1024));
	shadow.FillSolidRect(0,0,1024,1024,RGB(230,230,230));
	darkshadow.FillSolidRect(0,0,1024,1024,RGB(192,192,192));

	crosshair.CreateCompatibleDC(GetDC());
	SelectObject((HDC)crosshair, LoadImage(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_CROSSHAIR2), IMAGE_BITMAP, 0, 0, 0));
	crosshair.SelectStockObject(GRAY_BRUSH);

	buffer.SetBkMode(TRANSPARENT);
	buffer.SetTextColor(RGB(244,244,244));

	dlgBar = &m_wndDlgBar;

	saveprogwin.Create(IDD_SAVEPROGDLG, this);

	toolwin.Create(IDD_TOOLSDLG, this);
	toolwin.ShowWindow(SW_NORMAL);

	// give our editor windows pointers to the tileset bitmaps; they'll need them for stuff
	doorwin.tilesets = tilesets;
	destwin.tilesets = tilesets;

	for(i = 0; i < 80; i++) {
		for(int j = 0; j < 32; j++) {
			doorwin.gsets[i][j] = &global_sets[i][j];
			destwin.gsets[i][j] = &global_sets[i][j];
		}
	}

	for(i = 0; i < 320; i++) {
		for(int j = 0; j < 256; j++) {
			doorwin.lsets[i][j] = &local_sets[i][j];
			doorwin.map_tiles[i][j] = &map_tiles[i][j];
			destwin.lsets[i][j] = &local_sets[i][j];
			destwin.map_tiles[i][j] = &map_tiles[i][j];
		}
	}

	// set pretty animation timer
	SetTimer(0, anim_speed, NULL);

	auto_update = FALSE;
	m_wndDlgBar.SetDlgItemInt(IDC_EDIT_X, 0);
	m_wndDlgBar.SetDlgItemInt(IDC_EDIT_Y, 0);
	auto_update = TRUE;

	m_wndStatusBar.SetPaneInfo(1, 1024, SBPS_NOBORDERS, 0);
	m_wndStatusBar.SetPaneInfo(2, 1025, SBPS_NORMAL, 100);
	m_wndStatusBar.SetPaneInfo(3, 1026, SBPS_NORMAL, 100);

	m_wndStatusBar.SetPaneText(1, "");
	m_wndStatusBar.SetPaneText(2, "");
	m_wndStatusBar.SetPaneText(3, "");

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers


void CMainFrame::OnFileOpen() 
{
	// Display OpenFileName dialog, get filename,
	// Call LoadFile to load data
	OPENFILENAME ofn;       // common dialog box structure
	char szFile[260];       // buffer for filename

	szFile[0] = NULL;

	// Initialize OPENFILENAME
	ZeroMemory(&ofn, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFile = szFile;
	ofn.nMaxFile = sizeof(szFile);
	ofn.lpstrFilter = "SNES ROM files (*.smc)\0*.SMC\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = NULL;
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

	CFileDialog dlg(TRUE, "smc", NULL, OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST, "SNES ROM files (*.smc)\0*.SMC\0", this);

	dlg.m_ofn = ofn;

	// Display the Open dialog box
	int nResult = dlg.DoModal();

	if(nResult == IDOK) {
		if(CheckFile(szFile)) {
			LoadFile(szFile);
			theApp.AddToRecentFileList(szFile);
			FileName = szFile;
		}
	}
	//if(GetOpenFileName(&ofn)) {
	//	if(CheckFile(szFile)) {
	//		LoadFile(szFile);
	//		theApp.AddToRecentFileList(szFile);
	//	}
	//}
}

void CMainFrame::LoadFile(CString szFilename)
{
	RomFile = fopen(szFilename,"rb+");
	is_loaded = TRUE;

	// enable dialog bar controls
	dlgBar->GetDlgItem(IDC_EDIT_X)->EnableWindow();
	dlgBar->GetDlgItem(IDC_EDIT_Y)->EnableWindow();
	dlgBar->GetDlgItem(IDC_COMBO_TILESET)->EnableWindow();
	dlgBar->GetDlgItem(IDC_EDIT_PALETTE)->EnableWindow();
	dlgBar->GetDlgItem(IDC_EDIT_MUSIC)->EnableWindow();

	m_wndStatusBar.SetWindowText("Loading...");

	Load();

	m_wndStatusBar.SetWindowText("Ready");

	CMapEditView* mv = (CMapEditView*)GetActiveView();
	mv->SelectSector(0,0);

	CString str;
	str.LoadString(AFX_IDS_APP_TITLE);
	str.Insert(0, " - ");
	str.Insert(0, szFilename.Right(szFilename.GetLength()-szFilename.ReverseFind('\\')-1));

	SetWindowText(str);
}

void CMainFrame::CloseFile()
{
	//fclose(RomFile);
	_fcloseall();

	is_loaded = FALSE;

	dlgBar->GetDlgItem(IDC_EDIT_X)->EnableWindow(FALSE);
	dlgBar->GetDlgItem(IDC_EDIT_Y)->EnableWindow(FALSE);
	dlgBar->GetDlgItem(IDC_COMBO_TILESET)->EnableWindow(FALSE);
	dlgBar->GetDlgItem(IDC_EDIT_PALETTE)->EnableWindow(FALSE);
	dlgBar->GetDlgItem(IDC_EDIT_MUSIC)->EnableWindow(FALSE);

	for(int i = 0; i < 320; i++) {
		for(int j = 0; j < 256; j++) {
			map_tiles[i][j] = 0;
			local_sets[i][j] = 0;
		}
	}
	for(i = 0; i < 80; i++) {
		for(int j = 0; j < 32; j++) {
			global_sets[i][j] = 0;
			global_pal[i][j] = 0;
			global_music[i][j] = 0;
		}
	}
	for(i = 0; i < 1280; i++)
		object_areas[i].num_doors = 0;

	GetActiveView()->Invalidate(FALSE);

	CMapEditView* mv = (CMapEditView*)GetActiveView();
	mv->SelectSector(-1,-1);

	toolwin.gSet = -1;
	toolwin.Invalidate(FALSE);

	CString str;
	str.LoadString(AFX_IDS_APP_TITLE);
	SetWindowText(str);
}

void CMapEditDoc::OnFileSave() 
{
	POSITION pos = GetFirstViewPosition();

	CMainFrame *w = ((CMainFrame*)GetNextView(pos)->GetParent());
		
	w->m_wndStatusBar.SetPaneText(0, "Saving...");

	Save();

	w->m_wndStatusBar.SetPaneText(0, "Ready");
}

void CMapEditDoc::OnFileSaveAs() 
{
	OnFileSave();
}

BOOL IsCloseable()
{
	CString prompt;
	BOOL retVal = FALSE;

	if(is_loaded) {
		prompt.Format("Save changes to %s?", FileName);

		int exit = AfxMessageBox(prompt, MB_YESNOCANCEL);

		if(exit == IDYES) {
			Save();
			retVal = TRUE;
		}
		if(exit == IDNO) {
			retVal = TRUE;
		}
	}
	else
		retVal = TRUE;

	return retVal;
}


BOOL CMapEditDoc::CanCloseFrame(CFrameWnd* pFrame) 
{
	return IsCloseable();//CDocument::CanCloseFrame(pFrame);
}





void CMainFrame::OnFileClose() 
{
	if(IsCloseable())
		CloseFile();
}

void CMainFrame::OnUpdateFileClose(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(is_loaded);
}

void CMainFrame::OnUpdateFileSave(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(is_loaded);
}




void CMainFrame::OnEditCopy() 
{
	Copy();
}

void CMainFrame::OnEditPaste() 
{
	Paste();
	GetActiveView()->Invalidate(FALSE);
}


void CMainFrame::OnUpdateEditCopy(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(clipboard.isUseable && edit_mode == MODE_MAP && is_loaded);
}

void CMainFrame::OnUpdateEditPaste(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(clipboard.isValid && edit_mode == MODE_MAP && is_loaded);
}



void CMainFrame::OnEditUndo() 
{
	DoUndo();
	GetActiveView()->Invalidate(FALSE);
}

void CMainFrame::OnUpdateEditUndo(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(undoable && edit_mode == MODE_MAP && is_loaded);
}




// view stuff
void CMainFrame::OnViewSpriteborders() 
{
	spriteborder = !spriteborder;
	GetActiveView()->Invalidate(FALSE);
}

void CMainFrame::OnUpdateViewSpriteborders(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!sprite_outlines_only);
	pCmdUI->SetCheck(spriteborder);
}

void CMainFrame::OnViewGridlines() 
{
	gridlines = !gridlines;
	GetActiveView()->Invalidate(FALSE);
}

void CMainFrame::OnUpdateViewGridlines(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(gridlines);
}

void CMainFrame::OnViewSpriteOutlines() 
{
	sprite_outlines_only = !sprite_outlines_only;

	if(sprite_outlines_only)
		spriteborder = TRUE;

	GetActiveView()->Invalidate(FALSE);
}

void CMainFrame::OnUpdateViewSpriteOutlines(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(sprite_outlines_only);
	
}



void CMainFrame::OnViewDragSpriteOutline() 
{
	// TODO: Add your command handler code here
	drag_sprite_outline = !drag_sprite_outline;
}

void CMainFrame::OnUpdateViewDragSpriteOutline(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(drag_sprite_outline);
}

void CMainFrame::OnViewBorderFlash() 
{
	// TODO: Add your command handler code here
	color_anim = !color_anim;
}

void CMainFrame::OnUpdateViewBorderFlash(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(color_anim);
}




// MODE SELECTORS

void CMainFrame::OnModeMap() 
{
	edit_mode = MODE_MAP;

	toolwin.ShowWindow(SW_SHOWNORMAL);
	dlgBar->GetDlgItem(IDC_COMBO_TILESET)->EnableWindow();
	dlgBar->GetDlgItem(IDC_EDIT_PALETTE)->EnableWindow();
	dlgBar->GetDlgItem(IDC_EDIT_MUSIC)->EnableWindow();

	GetActiveView()->Invalidate(FALSE);
}

void CMainFrame::OnUpdateModeMap(CCmdUI* pCmdUI) 
{
	pCmdUI->SetRadio((edit_mode==MODE_MAP));

	// don't allow mode switch while seeking
	pCmdUI->Enable(door_mode != DOOR_DESTSEEK);
}

void CMainFrame::OnModeDoor() 
{
	edit_mode = MODE_DOOR;

	toolwin.ShowWindow(SW_HIDE);
	dlgBar->GetDlgItem(IDC_COMBO_TILESET)->EnableWindow(FALSE);
	dlgBar->GetDlgItem(IDC_EDIT_PALETTE)->EnableWindow(FALSE);
	dlgBar->GetDlgItem(IDC_EDIT_MUSIC)->EnableWindow(FALSE);

	GetActiveView()->Invalidate(FALSE);
}

void CMainFrame::OnUpdateModeDoor(CCmdUI* pCmdUI) 
{
	pCmdUI->SetRadio((edit_mode==MODE_DOOR));

	// don't allow mode switch while seeking
	pCmdUI->Enable(door_mode != DOOR_DESTSEEK);
}

void CMainFrame::OnModeSprite() 
{
	edit_mode = MODE_SPRITE;

	toolwin.ShowWindow(SW_HIDE);
	dlgBar->GetDlgItem(IDC_COMBO_TILESET)->EnableWindow(FALSE);
	dlgBar->GetDlgItem(IDC_EDIT_PALETTE)->EnableWindow(FALSE);
	dlgBar->GetDlgItem(IDC_EDIT_MUSIC)->EnableWindow(FALSE);

	GetActiveView()->Invalidate(FALSE);
}

void CMainFrame::OnUpdateModeSprite(CCmdUI* pCmdUI) 
{
	pCmdUI->SetRadio(edit_mode == MODE_SPRITE);

	// don't allow mode switch while seeking
	pCmdUI->Enable(door_mode != DOOR_DESTSEEK);
}





void CMainFrame::OnToolsAddDoor() 
{
	// TODO: Add your command handler code here
	m_wndStatusBar.SetWindowText("Click a location for the new door. (Right-click to cancel)");

	door_mode = DOOR_ADD;

	is_door_selected = FALSE;

	GetActiveView()->Invalidate(FALSE);
}

void CMainFrame::OnUpdateToolsAddDoor(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(edit_mode == MODE_DOOR && door_mode != DOOR_DESTSEEK && is_loaded);
	pCmdUI->SetRadio(door_mode == DOOR_ADD);
}

void CMainFrame::OnToolsAddSprite() 
{
	sprite_mode = SPRITE_ADD;

	m_wndStatusBar.SetWindowText("Click a location for the new sprite. (Right-click to cancel)");

	GetActiveView()->Invalidate(FALSE);
}

void CMainFrame::OnUpdateToolsAddSprite(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(edit_mode == MODE_SPRITE);
}


void CMainFrame::OnToolsRemoveObject() 
{
	// TODO: Add your command handler code here
	if(edit_mode == MODE_DOOR)
	{
		RemDoor(object_areas, sel_area, sel_door);
		is_door_selected = FALSE;
	}
	else if(edit_mode == MODE_SPRITE)
	{
		RemSprite(object_areas, sel_area, sel_sprite);
		is_sprite_selected = FALSE;
	}
	GetActiveView()->Invalidate(FALSE);

}

void CMainFrame::OnUpdateToolsRemoveObject(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(is_loaded && ((edit_mode == MODE_SPRITE && is_sprite_selected) || (edit_mode == MODE_DOOR && is_door_selected)));
}



void CMainFrame::OnToolsDestination() 
{
	((CMapEditView*)GetActiveView())->ShowDestinationWin();
}

void CMainFrame::OnUpdateToolsDestinationEdit(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(is_loaded && edit_mode == MODE_DOOR);
}


void CMainFrame::OnToolsTPTEditor() 
{
	// Show TPT editor window
	
}

void CMainFrame::OnUpdateToolsTPTEditor(CCmdUI* pCmdUI) 
{
	
}












void CMainFrame::OnTimer(UINT nIDEvent) 
{
	// TODO: Add your message handler code here and/or call default
	if(color_anim)
	{
		if(edit_mode == MODE_DOOR || (edit_mode == MODE_SPRITE && spriteborder))
		{
			timer_update = 1;
			GetActiveView()->Invalidate(FALSE);
		}
	}

	CFrameWnd::OnTimer(nIDEvent);
}





/////////////////////////////////////////////////////////////////////////////
// CMapEditView

IMPLEMENT_DYNCREATE(CMapEditView, CScrollView)

BEGIN_MESSAGE_MAP(CMapEditView, CScrollView)
	//{{AFX_MSG_MAP(CMapEditView)
	ON_WM_KEYDOWN()
	ON_WM_VSCROLL()
	ON_WM_HSCROLL()
	ON_WM_MOUSEWHEEL()
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_SETCURSOR()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMapEditView construction/destruction

CMapEditView::CMapEditView()
{
	// TODO: add construction code here

}

CMapEditView::~CMapEditView()
{
}

BOOL CMapEditView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMapEditView drawing

void CMapEditView::OnDraw(CDC* pDC)
{
	CRect rect;
	GetWindowRect(rect);
	int nx = (rect.Width()/32)+1;
	int ny = (rect.Height()/32)+1;

	for(int y = 0; y < ny; y++) {
		for(int x = 0; x < nx; x++) {
			if(x+xOffset >= 0 && y+yOffset >= 0 && x+xOffset < 256 && y+yOffset < 320) {
				buffer.BitBlt(x*32,y*32,32,32,&tilesets[global_sets[(y+yOffset)/4][(x+xOffset)/8]],map_tiles[y+yOffset][x+xOffset]*32,local_sets[y+yOffset][x+xOffset]*32,SRCCOPY);
				if(gridlines) {
					buffer.MoveTo(x*32,y*32);
					buffer.LineTo(x*32,y*32+32);
					buffer.MoveTo(x*32,y*32);
					buffer.LineTo(x*32+32,y*32);
				}
			}
			else
				buffer.BitBlt(x*32,y*32,32,32,NULL,0,0,BLACKNESS);
		}
	}

	if(edit_mode == MODE_MAP) {
		CString str;
		str.Format("Palette: %d Music: %d",global_pal[sector_y][sector_x],global_music[sector_y][sector_x]);
		buffer.BitBlt(sector_x*8*32-(xOffset*32)+1, sector_y*4*32-(yOffset*32)+1, 8*32, 4*32, &shadow, 0, 0, SRCAND);
		buffer.Draw3dRect(sector_x*8*32-(xOffset*32), sector_y*4*32-(yOffset*32), 8*32+1, 4*32+1, RGB(0,0,255),RGB(0,0,255));
		buffer.Draw3dRect(sector_x*8*32-(xOffset*32)+1, sector_y*4*32-(yOffset*32)+1, 8*32-1, 4*32-1, RGB(0,0,255),RGB(0,0,255));
		buffer.TextOut(sector_x*8*32-(xOffset*32)+5, sector_y*4*32-(yOffset*32)+5,str);
	}

	else if(edit_mode == MODE_DOOR) {
		int areax = GetScrollPos(SB_HORZ) / 32 / 8;
		int areay = GetScrollPos(SB_VERT) / 32 / 8;

		if(timer_update) {
			timer_update = 0;
			cur_frame++;
			if(cur_frame == num_frames) cur_frame = 0;
		}

		COLORREF clr = anim_clrs[cur_frame];//RGB(0,0,255);


		int sx, sy, drawsel = 0;
		for(int i = areax; i < areax+5; i++) {
			for(int j = areay; j < areay+5; j++) {
				int cur = j*32 + i;

				int row = cur/32;
				int col = cur-row*32;

				if(cur < 1280) {
					for(int k = 0; k < object_areas[cur].num_doors; k++) {
						int x = col*256+object_areas[cur].doors[k].x*8-GetScrollPos(SB_HORZ);
						int y = row*256+object_areas[cur].doors[k].y*8-GetScrollPos(SB_VERT);

						int w = 8;
						int h = 8;

						//if(object_areas[cur].doors[k].flags == 1) w = 16;
						
						if(drag_area == cur && drag_door == k && is_door_dragging)
						{
							buffer.Draw3dRect(x, y, w, h, clr, clr);
							buffer.InvertRect(CRect(x, y, x+w, y+h));
						}
						else if(sel_area == cur && sel_door == k && is_door_selected)
						{
							sx = x;
							sy = y;
							drawsel = 1;
						}
						else
							buffer.Draw3dRect(x, y, w, h, clr, clr);
					}
				}
			}
		}
		if(drawsel)
		{
			buffer.Draw3dRect(sx-1, sy-1, 8+2, 8+2, clr/RGB(2,2,2), clr/RGB(2,2,2));
			buffer.InvertRect(CRect(sx-1, sy-1, sx+8+1, sy+8+1));
		}

		if(door_mode == DOOR_ADD) {
			int x = int(mouse_x/8)*8;
			int y = int(mouse_y/8)*8;

			CPoint point;
			CRect rect;
			GetCursorPos(&point);
			GetWindowRect(rect);

			if(rect.PtInRect(point))
				buffer.Draw3dRect(x, y, 8, 8, clr, clr);
		}

		if(door_mode == DOOR_DESTSEEK) {
			int x = (mouse_x/8)*8;
			int y = (mouse_y/8)*8;

			buffer.InvertRect(CRect(0, y+3, 1280, y+4));
			buffer.InvertRect(CRect(x+3, 0, x+4, 1280));

			buffer.BitBlt(x-60, y-28, 128, 64, &crosshair, 0, 0, SRCAND);
			buffer.Draw3dRect(x, y, 8, 8, RGB(255,0,0), RGB(255,0,0));
		}
	}

	else if(edit_mode == MODE_SPRITE) {
		int areax = GetScrollPos(SB_HORZ) / 32 / 8;
		int areay = GetScrollPos(SB_VERT) / 32 / 8;

		if(timer_update) {
			timer_update = 0;
			cur_frame++;
			if(cur_frame == num_frames) cur_frame = 0;
		}

		COLORREF clr = anim_clrs[cur_frame];//RGB(0,0,255);

		int spr_count = 0;

		for(int i = areax; i < areax+5; i++) {
			for(int j = areay; j < areay+5; j++) {

				int cur = j*32 + i;

				int row = cur/32;
				int col = cur-row*32;

				if(cur < 1280) {
					for(int k = 0; k < object_areas[cur].num_sprites; k++) {
						int x2 = object_areas[cur].sprites[k].x;
						int y2 = object_areas[cur].sprites[k].y;

						int x = col*256+x2-GetScrollPos(SB_HORZ);
						int y = row*256+y2-GetScrollPos(SB_VERT);

						int tptnum = object_areas[cur].sprites[k].tptNum;
						int spr = sprConv[tpt[tptnum].sprite];

						int w = sprites[spr].width*8;
						int h = sprites[spr].height*8;

						x -= w/2;
						y -= h-8;

						//if(object_areas[cur].doors[k].flags == 1) w = 16;
						
						/*if(drag_area == cur && drag_door == k && is_door_dragging)
						{
							buffer.Draw3dRect(x, y, w, h, clr, clr);
							buffer.InvertRect(CRect(x, y, x+w, y+h));
						}
						else if(sel_area == cur && sel_door == k && is_door_selected)
						{
							sx = x;
							sy = y;
							drawsel = 1;
						}
						else*/


						if(is_sprite_dragging && drag_area == cur && drag_sprite == k)
						{
							if(spriteborder || drag_sprite_outline) {
								buffer.Draw3dRect(x-1, y-1, w+2, h+2, clr, clr);
								buffer.InvertRect(CRect(x-1, y-1, x+w+1, y+h+1));
								buffer.InvertRect(CRect(x, y, x+w, y+h));
							}

							if(!sprite_outlines_only && !drag_sprite_outline)
								sprites[spr].Draw(&buffer, x, y, pal[sprites[spr].palette]);
						}
						else if(is_sprite_selected && sel_area == cur && sel_sprite == k)
						{
							buffer.Draw3dRect(x-1, y-1, w+2, h+2, clr/RGB(2,2,2), clr/RGB(2,2,2));
							buffer.InvertRect(CRect(x-1, y-1, x+w+1, y+h+1));
							//buffer.InvertRect(CRect(x, y, x+w, y+h));


							if(!sprite_outlines_only && !drag_sprite_outline)
								sprites[spr].Draw(&buffer, x, y, pal[sprites[spr].palette]);
						}
						else
						{
							if(spriteborder)
								buffer.Draw3dRect(x-1, y-1, w+2, h+2, clr, clr);

							if(!sprite_outlines_only)
								sprites[spr].Draw(&buffer, x, y, pal[sprites[spr].palette]);
						}

						disp_spr[spr_count].area = cur;
						disp_spr[spr_count].spr = k;
						disp_spr[spr_count].x = x;
						disp_spr[spr_count].y = y;
						disp_spr[spr_count].w = w;
						disp_spr[spr_count++].h = h;
					}
				}
			}
		}
		n_disp_spr = spr_count;

		if(sprite_mode == SPRITE_ADD) {
			int x = mouse_x;
			int y = mouse_y;

			CPoint point;
			CRect rect;
			GetCursorPos(&point);
			GetWindowRect(rect);

			if(rect.PtInRect(point))
				buffer.Draw3dRect(x, y, 8, 8, clr, clr);
		}
	}


	// The buffer (a small portion of the total display surface, so to speak) is drawn
	// to the current scroll position on the DC.
	// In this manner, only the part of the screen currently being viewed actually gets drawn.
	// (this saves a hell of a lot of time when drawing on an 8192x10240 surface)

	pDC->BitBlt(GetScrollPos(SB_HORZ),GetScrollPos(SB_VERT),nx*32,ny*32,&buffer,0,0,SRCCOPY);


}

void CMapEditView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();

	CSize sizeTotal;

	sizeTotal.cx = 8224;
	sizeTotal.cy = 10272;
	SetScrollSizes(MM_TEXT, sizeTotal, CSize(0,0), CSize(0,0));
	SetScrollRange(SB_VERT,0,320);
	SetScrollRange(SB_HORZ,0,256);
}

/////////////////////////////////////////////////////////////////////////////
// CMapEditView diagnostics

#ifdef _DEBUG
void CMapEditView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CMapEditView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CMapEditDoc* CMapEditView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMapEditDoc)));
	return (CMapEditDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMapEditView message handlers

void CMapEditView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	switch(nChar) {
	case 37:	// LEFT
		xOffset--;
		break;
	case 38:	// UP
		yOffset--;
		break;
	case 39:	// RIGHT
		xOffset++;
		break;
	case 40:	// DOWN
		yOffset++;
		break;
	}
	ScrollTo(xOffset,yOffset);

	CScrollView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CMapEditView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	switch(nSBCode) {
	case SB_THUMBTRACK:
	case SB_THUMBPOSITION:
		yOffset = nPos/32;
		break;

	case SB_LINEUP:
		yOffset--;
		break;
	case SB_LINEDOWN:
		yOffset++;
		break;

	case SB_PAGEUP:
		yOffset-=8;
		break;
	case SB_PAGEDOWN:
		yOffset+=8;
		break;
	}
	ScrollTo(xOffset, yOffset);

	/*Invalidate(FALSE);

	if(yOffset > 301) yOffset = 301;
	if(yOffset < 0) yOffset = 0;

	SetScrollPos(SB_VERT, yOffset*32);
	dlgBar->SetDlgItemInt(IDC_EDIT_Y, yOffset);*/
}

void CMapEditView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	switch(nSBCode) {
	case SB_THUMBTRACK:
	case SB_THUMBPOSITION:
		xOffset = nPos/32;
		break;

	case SB_LINELEFT:
		xOffset--;
		break;
	case SB_LINERIGHT:
		xOffset++;
		break;

	case SB_PAGELEFT:
		xOffset-=8;
		break;
	case SB_PAGERIGHT:
		xOffset+=8;
		break;
	}
	ScrollTo(xOffset, yOffset);
	//Invalidate(FALSE);

	/*if(xOffset > 225) xOffset = 225;
	if(xOffset < 0) xOffset = 0;

	SetScrollPos(SB_HORZ, xOffset*32);
	dlgBar->SetDlgItemInt(IDC_EDIT_X, xOffset);*/
}


BOOL CMapEditView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	// Override cancels scrolling messages
	if(zDelta < 0) yOffset+=2;
	else if(zDelta > 0) yOffset-=2;

	ScrollTo(xOffset, yOffset);
	
	return FALSE;
}



void CMapEditView::ScrollTo(int x, int y)
{
	CRect rect;
	GetClientRect(rect);

	int xlim = 256 - rect.Width()/32;
	int ylim = 320 - rect.Height()/32;

	if(x > xlim) x = xlim;
	if(x < 0) x = 0;
	if(y > ylim) y = ylim;
	if(y < 0) y = 0;

	dlgBar->SetDlgItemInt(IDC_EDIT_X, x);
	dlgBar->SetDlgItemInt(IDC_EDIT_Y, y);

	Invalidate(FALSE);

	SetScrollPos(SB_VERT,y*32,TRUE);
	SetScrollPos(SB_HORZ,x*32,TRUE);
}


void CMapEditView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if(is_loaded)
	{
		if(edit_mode == MODE_MAP)
		{
			int xTile = point.x/32;
			int yTile = point.y/32;
			xTile+=xOffset;
			yTile+=yOffset;

			if(infomode)
			{
				infomode = FALSE;
				tilemode = -1;
			}

			switch(tilemode)
			{
			case TILE_PLACE:
				AddUndoState(xTile,yTile,1,1);
				local_sets[yTile][xTile] = place_set;
				map_tiles[yTile][xTile] = place_tile;
				Invalidate(FALSE);
				break;
			case TILE_PICK:
				place_set = local_sets[yTile][xTile];
				place_tile = map_tiles[yTile][xTile];
				toolwin.ScrollToTile(global_sets[yTile/4][xTile/8],place_set, place_tile);
				break;
			case TILE_INFO:
				infomode = TRUE;
				infoX = xTile;
				infoY = yTile;
				ShowInfo();
				break;
			case TILE_SELECT:
				break;
			default:
				tilemode = TILE_PLACE;
				Invalidate(FALSE);
				break;
			}
		}

		else if(edit_mode == MODE_DOOR)
		{
			// do door stuff instead
			if(door_mode == DOOR_NORMAL)
			{
				drag_area = ((point.y+GetScrollPos(SB_VERT))/32/8)*32+((point.x+GetScrollPos(SB_HORZ))/32/8);
				drag_door = -1;

				int x = ((point.x + GetScrollPos(SB_HORZ))%256)/8;
				int y = ((point.y + GetScrollPos(SB_VERT))%256)/8;

				for(int i = 0; i < object_areas[drag_area].num_doors; i++) {
					if(object_areas[drag_area].doors[i].x == x && object_areas[drag_area].doors[i].y == y) {
						drag_door = i;
						is_door_dragging = TRUE;
					}
				}

				if(is_door_dragging)
				{
					sel_area = drag_area;
					sel_door = drag_door;
					is_door_selected = TRUE;
				}
				else
					is_door_selected = FALSE;
			}

			
		}

		else if(edit_mode == MODE_SPRITE)
		{
			// okay, problem exists here.
			// if a sprite's upper bounds exceed the top of its respective area,
			// and that upper portion of the sprite is clicked, somehow it gets
			// bumped up to the next area...

			int x = point.x; //((point.x + GetScrollPos(SB_HORZ))%256);
			int y = point.y; //((point.y + GetScrollPos(SB_VERT))%256);

			//int area = ((point.y+GetScrollPos(SB_VERT))/32/8)*32+((point.x+GetScrollPos(SB_HORZ))/32/8);

			for(int i = 0; i < n_disp_spr; i++) {
				int sx = disp_spr[i].x;
				int sy = disp_spr[i].y;
				int sw = disp_spr[i].w;
				int sh = disp_spr[i].h;

				if(x >= sx && x <= sx+sw && y >= sy && y <= sy+sh) {
					drag_area = disp_spr[i].area;
					drag_sprite = disp_spr[i].spr;

					spr_offx = point.x - sx-(sw/2);
					spr_offy = point.y - sy-(sh-8);

					is_sprite_dragging = TRUE;
				}
			}

			if(is_sprite_dragging)
			{
				CString str;
				str.Format("a:%d s:%d (%d, %d)", drag_area, drag_sprite, spr_offx, spr_offy);
				((CMainFrame*)GetParent())->m_wndStatusBar.SetPaneText(2, str);

				sel_area = drag_area;
				sel_sprite = drag_sprite;
				is_sprite_selected = TRUE;
			}
			else
				is_sprite_selected = FALSE;
		}

		Invalidate(FALSE);
	}

	CScrollView::OnLButtonDown(nFlags, point);
}



void CMapEditView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	if(edit_mode == MODE_DOOR) {
		if(is_door_dragging) {

			int ar_y = drag_area / 32;
			int ar_x = drag_area - (ar_y*32);

			int ax = ((point.x + GetScrollPos(SB_HORZ))/8);		// absolute coordinates
			int ay = ((point.y + GetScrollPos(SB_VERT))/8);

			int x = ax-ar_x*32;									// relative coordinates
			int y = ay-ar_y*32;

			bool move_ok = true;

			ar_x = ax/32;
			ar_y = ay/32;

			int new_area = ar_y*32 + ar_x;

			for(int i = 0; i < object_areas[new_area].num_doors; i++) {
				if(object_areas[new_area].doors[i].x == ax%32 && object_areas[new_area].doors[i].y == ay%32)
					move_ok = false;
			}

			if(move_ok) {
				object_areas[drag_area].doors[drag_door].x = x;
				object_areas[drag_area].doors[drag_door].y = y;
			}

			Invalidate(FALSE);
		}

		if(door_mode == DOOR_ADD || door_mode == DOOR_DESTSEEK) {
			if(point.y/8 != mouse_y/8 || point.x/8 != mouse_x/8)
				Invalidate(FALSE);

			CString str;
			str.Format("%d, %d", (point.x+GetScrollPos(SB_HORZ))/8, (point.y+GetScrollPos(SB_VERT))/8);

			((CMainFrame*)GetParent())->m_wndStatusBar.SetPaneText(2, str);
		}
	}
	if(edit_mode == MODE_SPRITE)
	{
		if(is_sprite_dragging)
		{
			int ar_y = drag_area / 32;
			int ar_x = drag_area - (ar_y*32);

			int ax = ((point.x + GetScrollPos(SB_HORZ)));		// absolute coordinates
			int ay = ((point.y + GetScrollPos(SB_VERT)));

			int x = ax-ar_x*256;									// relative coordinates
			int y = ay-ar_y*256;


			object_areas[drag_area].sprites[drag_sprite].x = x - spr_offx;
			object_areas[drag_area].sprites[drag_sprite].y = y - spr_offy;

			InvalidateRect(CRect(point.x-128, point.y-128, point.x+128, point.y+128), FALSE);

			CString str;
			str.Format("a:%d s:%d (%d, %d)", drag_area, drag_sprite, x - spr_offx, y - spr_offy);
			((CMainFrame*)GetParent())->m_wndStatusBar.SetPaneText(2, str);

		}
		if(sprite_mode == SPRITE_ADD)
		{
			InvalidateRect(CRect(point.x-80, point.y-80, point.x+80, point.y+80), FALSE);

			CString str;
			str.Format("%d, %d", (point.x+GetScrollPos(SB_HORZ))/8, (point.y+GetScrollPos(SB_VERT))/8);

			((CMainFrame*)GetParent())->m_wndStatusBar.SetPaneText(2, str);
		}
	}

	mouse_x = point.x;
	mouse_y = point.y;
	
	CScrollView::OnMouseMove(nFlags, point);
}


void CMapEditView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	if(edit_mode == MODE_DOOR)
	{
		if(door_mode == DOOR_NORMAL)
		{
			if(is_door_dragging)
			{
				// move some stuff around
				int ar_y = drag_area / 32;
				int ar_x = drag_area - (ar_y*32);

				int ax = ((point.x + GetScrollPos(SB_HORZ))/8);		// absolute coordinates
				int ay = ((point.y + GetScrollPos(SB_VERT))/8);

				int x = ax-ar_x*32;									// relative coordinates
				int y = ay-ar_y*32;

				int dest_area = (ay/32)*32 + (ax/32);

				// compare the two areas; if they're the same, we don't need to do anything
				if(dest_area != drag_area)
				{
					// first, add a new door (yay :D)
					int dx = ax%32;
					int dy = ay%32;
					int dflags = object_areas[drag_area].doors[drag_door].flags;
					int ddest = object_areas[drag_area].doors[drag_door].destination;
					int ddestindex = object_areas[drag_area].doors[drag_door].destindex;
					int ddir = object_areas[drag_area].doors[drag_door].direction;
					int drope = object_areas[drag_area].doors[drag_door].ropetype;
					int desc = object_areas[drag_area].doors[drag_door].is_esc_end;
					AddDoor(object_areas, dest_area, dx, dy, dflags, ddest, ddestindex, ddir, drope, desc);
					RemDoor(object_areas, drag_area, drag_door);

					// if the door has moved, we also need to redo the selection
					sel_area = dest_area;
					sel_door = object_areas[dest_area].num_doors - 1;
					is_door_selected = TRUE;
				}

				is_door_dragging = FALSE;
				Invalidate(FALSE);
			}

			/*if(need_select) {
				// door selection
				sel_area = ((point.y+GetScrollPos(SB_VERT))/32/8)*32+((point.x+GetScrollPos(SB_HORZ))/32/8);
				sel_door = -1;

				int x = ((point.x + GetScrollPos(SB_HORZ))%256)/8;
				int y = ((point.y + GetScrollPos(SB_VERT))%256)/8;

				is_door_selected = FALSE;
				for(int i = 0; i < object_areas[drag_area].num_doors; i++) {
					if(object_areas[drag_area].doors[i].x == x && object_areas[drag_area].doors[i].y == y) {
						sel_door = i;
						is_door_selected = TRUE;
					}
				}
			}*/
		}

		if(door_mode == DOOR_DESTSEEK)
		{
			int x = (point.x + GetScrollPos(SB_HORZ))/8;
			int y = (point.y + GetScrollPos(SB_VERT))/8;

			// reopen our handy-dandy dealy
			destwin.seek_x = x;
			destwin.seek_y = y;

			((CMainFrame*)GetParent())->m_wndStatusBar.SetWindowText("Ready");
			((CMainFrame*)GetParent())->m_wndStatusBar.SetPaneText(2, "");

			ShowDestinationWin(FALSE);

			Invalidate(FALSE);
		}

		if(door_mode == DOOR_ADD)
		{
			int cur_area = ((point.y+GetScrollPos(SB_VERT))/32/8)*32+((point.x+GetScrollPos(SB_HORZ))/32/8);
			int x = ((point.x + GetScrollPos(SB_HORZ))%256)/8;
			int y = ((point.y + GetScrollPos(SB_VERT))%256)/8;

			AddDoor(object_areas, cur_area, x, y, 2, 0, 0, 0, 0, 0);

			door_mode = DOOR_NORMAL;
			((CMainFrame*)GetParent())->m_wndStatusBar.SetWindowText("Ready");
			((CMainFrame*)GetParent())->m_wndStatusBar.SetPaneText(2, "");
			Invalidate(FALSE);
		}
	}

	if(edit_mode == MODE_SPRITE)
	{
		if(is_sprite_dragging) {
			int ar_y = drag_area / 32;
			int ar_x = drag_area - (ar_y*32);

			int ax = ((point.x + GetScrollPos(SB_HORZ)));		// absolute coordinates
			int ay = ((point.y + GetScrollPos(SB_VERT)));

			int x = ax-ar_x*256;									// relative coordinates
			int y = ay-ar_y*256;

			int dest_area = (ay/256)*32 + (ax/256);

			if(dest_area != drag_area)
			{
				int sx = (ax%256) - spr_offx;
				int sy = (ay%256) - spr_offy;
				int stptNum = object_areas[drag_area].sprites[drag_sprite].tptNum;
				AddSprite(object_areas, dest_area, sx, sy, stptNum);
				RemSprite(object_areas, drag_area, drag_sprite);

				sel_area = dest_area;
				sel_sprite = object_areas[dest_area].num_sprites - 1;
				is_sprite_selected = TRUE;
			}

			is_sprite_dragging = FALSE;
			Invalidate(FALSE);
		}

		if(sprite_mode == SPRITE_ADD)
		{
			int cur_area = ((point.y+GetScrollPos(SB_VERT))/32/8)*32+((point.x+GetScrollPos(SB_HORZ))/32/8);
			int x = ((point.x + GetScrollPos(SB_HORZ))%256);
			int y = ((point.y + GetScrollPos(SB_VERT))%256);

			AddSprite(object_areas, cur_area, x, y, 0);

			sprite_mode = SPRITE_NORMAL;
			((CMainFrame*)GetParent())->m_wndStatusBar.SetWindowText("Ready");
			((CMainFrame*)GetParent())->m_wndStatusBar.SetPaneText(2, "");
			Invalidate(FALSE);
		}
	}


	CScrollView::OnLButtonUp(nFlags, point);
}

void CMapEditView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	if(edit_mode == MODE_DOOR) {
		int x = ((point.x + GetScrollPos(SB_HORZ))%256)/8;
		int y = ((point.y + GetScrollPos(SB_VERT))%256)/8;

		int area = ((point.y+GetScrollPos(SB_VERT))/32/8)*32+((point.x+GetScrollPos(SB_HORZ))/32/8);

		for(int i = 0; i < object_areas[area].num_doors; i++) {
			if(object_areas[area].doors[i].x == x && object_areas[area].doors[i].y == y) {

				//for(int j = 0; j < 1280; j++)
				//	*doorwin.areas[j].doors = *object_areas[j].doors;

				doorwin.dest = dest;

				doorwin.n_area = area;
				doorwin.n_door = i;
				doorwin.door = object_areas[area].doors[i];

				int nResult = doorwin.DoModal();

				if(nResult == 42)
				{
					reopen_doordlg = TRUE;	// set this flag so the door editor
										   // gets opened again when the destination
										  // editor closes
					destwin.sel_dest = doorwin.door.destindex;
					ShowDestinationWin(TRUE);
				}
				else if(nResult == IDOK)
				{
					object_areas[area].doors[i] = doorwin.door;
					Invalidate(FALSE);
				}
			}
		}
	}
	if(edit_mode == MODE_SPRITE)
	{
		int x = point.x; //((point.x + GetScrollPos(SB_HORZ))%256);
		int y = point.y; //((point.y + GetScrollPos(SB_VERT))%256);

		int area = ((point.y+GetScrollPos(SB_VERT))/32/8)*32+((point.x+GetScrollPos(SB_HORZ))/32/8);

		for(int i = 0; i < n_disp_spr; i++) {
			int sx = disp_spr[i].x;
			int sy = disp_spr[i].y;
			int sw = disp_spr[i].w;
			int sh = disp_spr[i].h;

			if(x >= sx && x <= sx+sw && y >= sy && y <= sy+sh) {
				int area = disp_spr[i].area;
				int spr = disp_spr[i].spr;
				spritewin.n_area = area;
				spritewin.n_sprite = spr;

				spritewin.sprite = object_areas[area].sprites[spr];

				for(int j = 0; j < 1583; j++)
					spritewin.tpt[j] = tpt[j];

				int nResult = spritewin.DoModal();

				if(nResult == IDOK) {
					object_areas[area].sprites[spr] = spritewin.sprite;
					for(int j = 0; j < 1583; j++)
						tpt[j] = spritewin.tpt[j];
					Invalidate(FALSE);
				}
			}
		}
	}

		/*for(int i = 0; i < object_areas[area].num_sprites; i++) {
			int sx = object_areas[area].sprites[i].x;
			int sy = object_areas[area].sprites[i].y;
			int w = sprites[sprConv[tpt[object_areas[area].sprites[i].tptNum].sprite]].width*8;
			int h = sprites[sprConv[tpt[object_areas[area].sprites[i].tptNum].sprite]].height*8;

			int x2 = x - (w/2);
			int y2 = y - h - 8;
			//sx-=w/2;
			//sy-=h-8;

			if(x2 >= sx && x2 <= sx+w && y >= sy && y <= sy+h) {

				spritewin.n_area = area;
				spritewin.n_sprite = i;

				spritewin.sprite = object_areas[area].sprites[i];

				for(int j = 0; j < 1582; j++)
					spritewin.tpt[j] = tpt[j];

				int nResult = spritewin.DoModal();

				if(nResult == IDOK) {
					object_areas[area].sprites[i] = spritewin.sprite;
					for(int j = 0; j < 1582; j++)
						tpt[j] = spritewin.tpt[j];
					Invalidate(FALSE);
				}
				//break;
			}
		}
	}*/

	CScrollView::OnLButtonDblClk(nFlags, point);
}



void CMapEditView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	if(is_loaded) {
		if(edit_mode == MODE_MAP) {
			SelectSector(((point.x/32)+xOffset)/8, ((point.y/32)+yOffset)/4);
			CScrollView::OnRButtonDown(nFlags, point);
		}
		if(edit_mode == MODE_DOOR)
		{
			if(door_mode == DOOR_DESTSEEK)
			{
				((CMainFrame*)GetParent())->m_wndStatusBar.SetWindowText("Ready");
				SendMessage(WM_SETCURSOR);

				ShowDestinationWin(FALSE);

				Invalidate(FALSE);
			}
			if(door_mode == DOOR_ADD)
			{
				door_mode = DOOR_NORMAL;
				((CMainFrame*)GetParent())->m_wndStatusBar.SetWindowText("Ready");
				Invalidate(FALSE);
			}
		}
	}
}





BOOL CMapEditView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	if(nHitTest == HTHSCROLL || nHitTest == HTVSCROLL)
		return CScrollView::OnSetCursor(pWnd, nHitTest, message);

	if(edit_mode == MODE_MAP) {
		if(nHitTest != HTCLIENT)
			return CScrollView::OnSetCursor(pWnd, nHitTest, message);

		switch(tilemode) {
		case TILE_PLACE:
			SetCursor(LoadCursor(NULL, IDC_ARROW));
			break;
		case TILE_PICK:
			SetCursor(LoadCursor(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDC_POINTER_COPY)));
			break;
		case TILE_INFO:
			SetCursor(LoadCursor(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDC_POINTER_INFO)));
			break;
		}
	}
	else if(edit_mode == MODE_DOOR) {
		if(nHitTest != HTCLIENT)
			return CScrollView::OnSetCursor(pWnd, nHitTest, message);

		switch(door_mode) {
		case DOOR_NORMAL:
			SetCursor(LoadCursor(NULL, IDC_ARROW));
			break;
		case DOOR_ADD:
			SetCursor(LoadCursor(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDC_POINTER_COPY)));
			break;
		case DOOR_DESTSEEK:
			SetCursor(0);
			break;
		}
	}
	else
		return CScrollView::OnSetCursor(pWnd, nHitTest, message);

	return TRUE;
}


void CMapEditView::ShowDestinationWin(bool bData)
{
	// show the destination editor dialog and handle destination seeking
	if(bData)
	{
		destwin.dest.entries = new DestinationEntry[dest.num_dest];
		destwin.dest.num_dest = dest.num_dest;
		for(int i = 0; i < dest.num_dest; i++)
			destwin.dest.entries[i] = dest.entries[i];
		for(i = 0; i < 1280; i++)
			destwin.areas[i] = object_areas[i];
	}

	if(door_mode == DOOR_DESTSEEK)
	{
		door_mode = DOOR_NORMAL;
		gridlines = react_gridlines;
		Invalidate(FALSE);
	}

	int nResult = destwin.DoModal();

	if(nResult == 42)
	{
		door_mode = DOOR_DESTSEEK;

		int x = (destwin.seek_x / 4);
		int y = (destwin.seek_y / 4);

		CRect rect;
		GetWindowRect(rect);

		ScrollTo(x-(rect.Width()-20)/64,y-(rect.Height()-50)/64);

		// gridlines will be turned back on after seeking
		react_gridlines = gridlines;
		gridlines = FALSE;

		Invalidate(FALSE);

		SetCursorPos((destwin.seek_x*8)-GetScrollPos(SB_HORZ)+rect.left+4, (destwin.seek_y*8)-GetScrollPos(SB_VERT)+rect.top+4);

		((CMainFrame*)GetParent())->m_wndStatusBar.SetWindowText("Click a location on the map to set the new destination coordinates (Right-click to cancel)");
	}

	else if(nResult == IDOK)
	{
		// user hit "okay", save the data
		dest.entries = new DestinationEntry[dest.num_dest];
		dest.num_dest = destwin.dest.num_dest;

		for(int i = 0; i < dest.num_dest; i++)
			dest.entries[i] = destwin.dest.entries[i];

		for(i = 0; i < 1280; i++)
			object_areas[i] = destwin.areas[i];
	}

	if(reopen_doordlg && nResult != 42) {
		reopen_doordlg = FALSE;

		if(nResult == IDOK) {
			doorwin.door.destindex = destwin.sel_dest;
			doorwin.dest = dest;
		}

		int nResult = doorwin.DoModal();

		if(nResult == 42) {
			// open the destination dialog yet again
			reopen_doordlg = TRUE;
			destwin.sel_dest = doorwin.door.destindex;
			ShowDestinationWin(FALSE);
		}
		if(nResult == IDOK) {
			object_areas[doorwin.n_area].doors[doorwin.n_door] = doorwin.door;
		}
		if(nResult != 42) {
			gridlines = react_gridlines;
			Invalidate(FALSE);
		}
	}

	// reset destination editor selection
	destwin.sel_dest = 0;
}


void CMapEditView::ShowInfo()
{
	CInfoDlg dlg;
	dlg.DoModal();
}

void CMapEditView::SelectSector(int x, int y)
{
	sector_x = x;
	sector_y = y;

	Invalidate(FALSE);

	CComboBox* CBTileset = (CComboBox*)dlgBar->GetDlgItem(IDC_COMBO_TILESET);

	CBTileset->SetCurSel(global_sets[y][x]);

	dlgBar->SetDlgItemInt(IDC_EDIT_X2, x);
	dlgBar->SetDlgItemInt(IDC_EDIT_Y2, y);
	dlgBar->SetDlgItemInt(IDC_EDIT_PALETTE, global_pal[y][x]);
	dlgBar->SetDlgItemInt(IDC_EDIT_MUSIC, global_music[y][x]);

	toolwin.gSet = global_sets[y][x];
	toolwin.Invalidate(FALSE);

	clipboard.isUseable = TRUE;
}


void CMainFrame::OnSelectTileset()
{
	CComboBox* CBTileset = (CComboBox*)dlgBar->GetDlgItem(IDC_COMBO_TILESET);
	global_sets[sector_y][sector_x] = CBTileset->GetCurSel();
	GetActiveView()->Invalidate(FALSE);
}

void CMainFrame::OnEditX()
{
	if(auto_update) {
		auto_update = FALSE;
		xOffset = dlgBar->GetDlgItemInt(IDC_EDIT_X);

		GetActiveView()->Invalidate(FALSE);
		GetActiveView()->SetScrollPos(SB_HORZ,xOffset*32);

		auto_update = TRUE;
	}
}

void CMainFrame::OnEditY()
{
	if(auto_update) {
		auto_update = FALSE;
		yOffset = dlgBar->GetDlgItemInt(IDC_EDIT_Y);

		GetActiveView()->Invalidate(FALSE);
		GetActiveView()->SetScrollPos(SB_VERT,yOffset*32);

		auto_update = TRUE;
	}
}

void CMainFrame::OnEditPalette()
{
	global_pal[sector_y][sector_x] = dlgBar->GetDlgItemInt(IDC_EDIT_PALETTE);
	GetActiveView()->Invalidate(FALSE);
}


void CMainFrame::OnEditMusic()
{
	global_music[sector_y][sector_x] = dlgBar->GetDlgItemInt(IDC_EDIT_MUSIC);
	GetActiveView()->Invalidate(FALSE);
}










/////////////////////////////////////////////////////////////////////////////
// CToolsDlg dialog


CToolsDlg::CToolsDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CToolsDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CToolsDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CToolsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CToolsDlg)
	DDX_Control(pDX, IDC_GROUP, m_GroupBox);
	DDX_Control(pDX, IDC_COMBO_LOCALSET, m_Combo_Localset);
	DDX_Control(pDX, IDC_SCROLLBAR, m_Scroll);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CToolsDlg, CDialog)
	//{{AFX_MSG_MAP(CToolsDlg)
	ON_WM_KEYDOWN()
	ON_WM_VSCROLL()
	ON_WM_LBUTTONDOWN()
	ON_WM_PAINT()
	ON_CBN_SELCHANGE(IDC_COMBO_LOCALSET, OnSelectLocalSet)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CToolsDlg message handlers

BOOL CToolsDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	scrollPos = 0;
	lSet = 0;
	gSet = -1;

	m_Scroll.SetScrollRange(0, 56, FALSE);

	buffer.CreateCompatibleDC(GetDC());
	SelectObject((HDC)buffer, CreateCompatibleBitmap((HDC)*GetDC(),150,280));

	m_Combo_Localset.SetCurSel(0);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}



void CToolsDlg::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{

	//CDialog::OnKeyDown(nChar, nRepCnt, nFlags);
}

BOOL CToolsDlg::PreTranslateMessage(MSG* pMsg) 
{
	if(pMsg->message == WM_KEYDOWN) {
		CMainFrame* mainfrm = (CMainFrame*)GetParent();
		mainfrm->GetActiveView()->SendMessage(pMsg->message, pMsg->wParam, pMsg->lParam);
		return TRUE;
	}

	return CDialog::PreTranslateMessage(pMsg);
}

void CToolsDlg::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	if(nSBCode == SB_THUMBPOSITION || nSBCode == SB_THUMBTRACK)
		scrollPos = nPos;

	switch(nSBCode) {
	case SB_PAGEDOWN:
	case SB_LINEDOWN:
		scrollPos++;
		break;
	case SB_PAGEUP:
	case SB_LINEUP:
		scrollPos--;
		break;
	}

	if(scrollPos < 0) scrollPos = 0;
	if(scrollPos > 56) scrollPos = 56;

	m_Scroll.SetScrollPos(scrollPos);
	Invalidate(FALSE);
}


void CToolsDlg::OnLButtonDown(UINT nFlags, CPoint point) 
{
	int x = (point.x-9)/33;
	int y = (point.y-45)/33;

	sel_x = x;
	sel_y = y;

	int tile = x+(y*4)+scrollPos*4;
	place_tile = tile;

	Invalidate(FALSE);

	CDialog::OnLButtonDown(nFlags, point);
}

void CToolsDlg::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

	buffer.FillSolidRect(0,0,150,280,RGB(0,0,0));
	if(gSet != -1) {
		for(int x = 0; x < 4; x++) {
			for(int y = 0; y < 8; y++) {
				int cx = ((y*4)+x+scrollPos*4)*32;
				
				//buffer.Draw3dRect(x*32,y*32,33,33,RGB(0,0,0),RGB(0,0,0));

				buffer.BitBlt(x*33+1,y*33+1,32,32,&tilesets[gSet],cx,lSet*32,SRCCOPY);
				//buffer.Draw3dRect(x*32,y*32,33,33,RGB(0,0,0),RGB(0,0,0));
					//buffer.MoveTo(x*32,y*32);
					//buffer.LineTo(x*32,y*32+32);
					//buffer.MoveTo(x*32,y*32);
					//buffer.LineTo(x*32+32,y*32);
			}
		}
	}
	buffer.Draw3dRect(sel_x*33+1,sel_y*33+1,32,32,RGB(45,45,244),RGB(45,45,244));

	dc.BitBlt(9,45,4*33+1,8*33+1,&buffer,0,0,SRCCOPY);

	// Do not call CDialog::OnPaint() for painting messages
}

void CToolsDlg::OnCancel()
{
	// override to prevent box from being closed
}
void CToolsDlg::OnOK()
{
	// ditto
}


void CToolsDlg::OnSelectLocalSet() 
{
	place_set = m_Combo_Localset.GetCurSel();
	lSet = m_Combo_Localset.GetCurSel();
	Invalidate(FALSE);
}


void CToolsDlg::ScrollToTile(int g, int l, int tile)
{
	m_Combo_Localset.SetCurSel(l);
	lSet = l;
	gSet = g;

	m_Scroll.SetScrollPos(tile/4);
	scrollPos = tile/4;

	sel_x = tile%4;
	sel_y = 0;

	Invalidate(FALSE);
}




void CMainFrame::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if(nChar == 16 && (tilemode != TILE_PICK)) {
		tilemode = TILE_PICK;
		GetActiveView()->SendMessage(WM_SETCURSOR, 0, (LPARAM)HTCLIENT);
	}
	if(nChar == 17 && (tilemode != TILE_INFO)) {
		tilemode = TILE_INFO;
		GetActiveView()->SendMessage(WM_SETCURSOR, 0, (LPARAM)HTCLIENT);
	}

	CFrameWnd::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CMainFrame::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if(nChar == 16 || nChar == 17) {
		tilemode = TILE_PLACE;
		GetActiveView()->SendMessage(WM_SETCURSOR, 0, (LPARAM)HTCLIENT);
	}

	CFrameWnd::OnKeyUp(nChar, nRepCnt, nFlags);
}

BOOL CMapEditApp::PreTranslateMessage(MSG* pMsg) 
{
	// Send all key messages to our frame window
	if(pMsg->message == WM_KEYDOWN || pMsg->message == WM_KEYUP || pMsg->message == WM_CHAR) {
		m_pMainWnd->SendMessage(pMsg->message, pMsg->wParam, pMsg->lParam);
	}

	return CWinApp::PreTranslateMessage(pMsg);
}





/////////////////////////////////////////////////////////////////////////////
// CInfoDlg dialog


CInfoDlg::CInfoDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CInfoDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CInfoDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CInfoDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CInfoDlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CInfoDlg, CDialog)
	//{{AFX_MSG_MAP(CInfoDlg)
	ON_WM_LBUTTONDOWN()
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CInfoDlg message handlers

BOOL CInfoDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	SetCapture();

	SetDlgItemInt(IDC_TILE, map_tiles[infoY][infoX]);
	SetDlgItemInt(IDC_LSET, local_sets[infoY][infoX]);
	SetDlgItemInt(IDC_GSET, global_sets[infoY/4][infoX/8]);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CInfoDlg::OnLButtonDown(UINT nFlags, CPoint point) 
{
	ReleaseCapture();
	infomode = FALSE;
	OnCancel();
	CDialog::OnLButtonDown(nFlags, point);
}

void CInfoDlg::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

	dc.FillSolidRect(2,2,68,68,RGB(0,0,0));
	dc.StretchBlt(4,4,64,64,&tilesets[global_sets[infoY/4][infoX/8]], map_tiles[infoY][infoX]*32, local_sets[infoY][infoX]*32, 32, 32, SRCCOPY);
}




































